// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


module debouncer #(parameter CNTR_WIDTH = 8) (
    input   logic pb_in,
    input   logic clk,
    output  logic out,
    output  logic pb_out
);

    bit local_input;
    bit [CNTR_WIDTH-1:0] cntr;
    
    debouncer_sync #(.ASYNC_INPUT_ACTIVE_HI(1'b0))
    u_debouncer_sync (
        .clk            (clk),
        .async_input    (pb_in),
        .sync_input     (local_input)
    );

 
    always_ff @ (posedge clk or posedge local_input) begin
        if (local_input) begin
            out     <= 1'b1;
            pb_out  <= 1'b0;
            cntr    <= {CNTR_WIDTH{1'b0}};
        end else begin
            out    <=  ~cntr[CNTR_WIDTH-1];
            pb_out <=  cntr[CNTR_WIDTH-1];
            if (out) begin
                cntr <= cntr + 1'b1;
            end
        end
    end

endmodule


module debouncer_sync #(parameter logic ASYNC_INPUT_ACTIVE_HI = 1'b1)
(
    input   logic clk,
    input   logic async_input,
    output  logic sync_input
);


    logic [1:0] input_pipe;
    logic active_high_input;

    assign active_high_input = ASYNC_INPUT_ACTIVE_HI ? async_input : ~async_input;
    always_ff @(posedge clk or posedge active_high_input) begin
        if (active_high_input) begin
            input_pipe <= 2'b11;
        end else begin
            input_pipe <= {input_pipe[0], 1'b0};
        end
    end

    assign sync_input = input_pipe[1];

endmodule
