// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//
// This ROM stores the field valuemask for PLL that need reconfig
// This file will not changed across A10 device parts 
//
module mr_rom_pll_valuemask_16bpc #(
    parameter NUMBER_OF_MIF_RANGE          = 6,
    parameter ROM_DEPTH_FOR_EACH_MIF_RANGE = 7,			      
    parameter ROM_SIZE                     = 8,   
    parameter TOTAL_ROM_DEPTH              = 128, // 6*14
    parameter ADDR_WIDTH                   = 7   // alt_clogb2(84) 
) (
    input  wire                  clock,
    input  wire [ADDR_WIDTH-1:0] addr_ptr,
    output wire [ROM_SIZE-1:0]   rdata_out
);

reg  [ROM_SIZE-1:0]   ROM [0:TOTAL_ROM_DEPTH-1];
wire [ROM_SIZE-1:0]   DATAA = {ROM_SIZE{1'b0}};
wire [ADDR_WIDTH-1:0] RADDR;
   
initial begin
            // FIELD_VALMASK
    // ROM OFFSET 0 (25MHz - 50MHz)  
    ROM[0] <= 8'b00001100;        //M_HIGH
    ROM[1] <= 8'b00001100;        //M_LOW
    ROM[2] <= 8'b00000000;        //N_HIGH
    ROM[3] <= 8'b00000000;        //N_LOW
    ROM[4] <= 8'b01000001;        //N_BYPASS_EN, N_EVEN_DUTY_EN
    ROM[5] <= 8'b00011000;        //C0_HIGH
    ROM[6] <= 8'b00011000;        //C0_LOW
    ROM[7] <= 8'b00110000;        //C1_HIGH
    ROM[8] <= 8'b00110000;        //C1_LOW
    ROM[9] <= 8'b00000000;        //C1 Even Duty Enable
    ROM[10] <= 8'b01000001;        //CHARGE_PUMP_CURRENT [2:0]
    ROM[11] <= 8'b00000000;        //CHARGE_PUMP_CURRENT [5:3]
    ROM[12] <= 8'b00100000;        //BANDWIDTH_CONTROL
    // ROM OFFSET 1 (51MHz - 100MHz)  
    ROM[13] <= 8'b00000110;        //M_HIGH
    ROM[14] <= 8'b00000110;        //M_LOW
    ROM[15] <= 8'b00000000;        //N_HIGH
    ROM[16] <= 8'b00000000;        //N_LOW
    ROM[17] <= 8'b00000001;        //N_BYPASS_EN, N_EVEN_DUTY_EN
    ROM[18] <= 8'b00001100;        //C0_HIGH
    ROM[19] <= 8'b00001100;        //C0_LOW
    ROM[20] <= 8'b00011000;        //C1_HIGH
    ROM[21] <= 8'b00011000;        //C1_LOW
    ROM[22] <= 8'b00000000;        //C1 Even Duty Enable
    ROM[23] <= 8'b01000001;        //CHARGE_PUMP_CURRENT [2:0]
    ROM[24] <= 8'b00000000;        //CHARGE_PUMP_CURRENT [5:3]
    ROM[25] <= 8'b00011000;        //BANDWIDTH_CONTROL
    // ROM OFFSET 2 (101MHz - 200MHz)  
    ROM[26] <= 8'b00000011;        //M_HIGH
    ROM[27] <= 8'b00000011;        //M_LOW
    ROM[28] <= 8'b00000000;        //N_HIGH
    ROM[29] <= 8'b00000000;        //N_LOW
    ROM[30] <= 8'b00000001;        //N_BYPASS_EN, N_EVEN_DUTY_EN
    ROM[31] <= 8'b00000110;        //C0_HIGH
    ROM[32] <= 8'b00000110;        //C0_LOW
    ROM[33] <= 8'b00001100;        //C1_HIGH
    ROM[34] <= 8'b00001100;        //C1_LOW
    ROM[35] <= 8'b00000000;        //C1 Even Duty Enable
    ROM[36] <= 8'b01000001;        //CHARGE_PUMP_CURRENT[2:0]
    ROM[37] <= 8'b00000000;        //CHARGE_PUMP_CURRENT[5:3]
    ROM[38] <= 8'b00011000;        //BANDWIDTH_CONTROL
    // ROM OFFSET 3 (201MHz - 300MHz)  
    ROM[39] <= 8'b00000010;        //M_HIGH
    ROM[40] <= 8'b00000010;        //M_LOW
    ROM[41] <= 8'b00000000;        //N_HIGH
    ROM[42] <= 8'b00000000;        //N_LOW
    ROM[43] <= 8'b01010001;        //N_BYPASS_EN, N_EVEN_DUTY_EN
    ROM[44] <= 8'b00000100;        //C0_HIGH
    ROM[45] <= 8'b00000100;        //C0_LOW
    ROM[46] <= 8'b00001000;        //C1_HIGH
    ROM[47] <= 8'b00001000;        //C1_LOW
    ROM[48] <= 8'b00000000;        //C1 Even Duty Enable
    ROM[49] <= 8'b00110001;        //CHARGE_PUMP_CURRENT[2:0]
    ROM[50] <= 8'b00000000;        //CHARGE_PUMP_CURRENT[5:3]
    ROM[51] <= 8'b00011000;        //BANDWIDTH_CONTROL
    // ROM OFFSET 4 (301MHz - 340MHz)  
    ROM[52] <= 8'b00000010;        //M_HIGH
    ROM[53] <= 8'b00000010;        //M_LOW
    ROM[54] <= 8'b00000001;        //N_HIGH
    ROM[55] <= 8'b00000001;        //N_LOW
    ROM[56] <= 8'b01010000;        //N_BYPASS_EN, N_EVEN_DUTY_EN
    ROM[57] <= 8'b00000010;        //C0_HIGH
    ROM[58] <= 8'b00000010;        //C0_LOW
    ROM[59] <= 8'b00000100;        //C1_HIGH
    ROM[60] <= 8'b00000100;        //C1_LOW
    ROM[61] <= 8'b00000000;        //C1 Even Duty Enable
    ROM[62] <= 8'b00110000;        //CHARGE_PUMP_CURRENT[2:0]
    ROM[63] <= 8'b00000000;        //CHARGE_PUMP_CURRENT[5:3]
    ROM[64] <= 8'b00011000;        //BANDWIDTH_CONTROL
    // ROM OFFSET 5 (85MHz - 150MHz) - 2.0  
    ROM[65] <= 8'b00000100;        //M_HIGH
    ROM[66] <= 8'b00000100;        //M_LOW
    ROM[67] <= 8'b00000000;        //N_HIGH
    ROM[68] <= 8'b00000000;        //N_LOW
    ROM[69] <= 8'b01110001;        //N_BYPASS_EN, N_EVEN_DUTY_EN
    ROM[70] <= 8'b00000010;        //C0_HIGH:4
    ROM[71] <= 8'b00000010;        //C0_LOW
    ROM[72] <= 8'b00000100;        //C1_HIGH
    ROM[73] <= 8'b00000100;        //C1_LOW
    ROM[74] <= 8'b00000000;        //C1 Even Duty Enable
    ROM[75] <= 8'b01110001;        //CHARGE_PUMP_CURRENT[2:0]
    ROM[76] <= 8'b00000000;        //CHARGE_PUMP_CURRENT[5:3]
    ROM[77] <= 8'b00011000;        //BANDWIDTH_CONTROL:3
    // Set the rest to all zeros
    ROM[78] <= 8'b000000000;
    ROM[79] <= 8'b000000000;
    ROM[80] <= 8'b000000000;
    ROM[81] <= 8'b000000000;
    ROM[82] <= 8'b000000000;
    ROM[83] <= 8'b000000000;
    ROM[84] <= 8'b000000000;
    ROM[84] <= 8'b000000000;
    ROM[85] <= 8'b000000000;
    ROM[86] <= 8'b000000000;
    ROM[87] <= 8'b000000000;
    ROM[88] <= 8'b000000000;
    ROM[89] <= 8'b000000000;
    ROM[90] <= 8'b000000000;
    ROM[91] <= 8'b000000000;
    ROM[92] <= 8'b000000000;
    ROM[93] <= 8'b000000000;
    ROM[94] <= 8'b000000000;
    ROM[95] <= 8'b000000000;
    ROM[96] <= 8'b000000000;
    ROM[97] <= 8'b000000000;
    ROM[98] <= 8'b000000000;
    ROM[99] <= 8'b000000000;
    ROM[100] <= 8'b000000000;
    ROM[101] <= 8'b000000000;
    ROM[102] <= 8'b000000000;
    ROM[103] <= 8'b000000000;
    ROM[104] <= 8'b000000000;
    ROM[105] <= 8'b000000000;
    ROM[106] <= 8'b000000000;
    ROM[107] <= 8'b000000000;
    ROM[108] <= 8'b000000000;
    ROM[109] <= 8'b000000000;
    ROM[110] <= 8'b000000000;
    ROM[111] <= 8'b000000000;
    ROM[112] <= 8'b000000000;
    ROM[113] <= 8'b000000000;
    ROM[114] <= 8'b000000000;
    ROM[115] <= 8'b000000000;
    ROM[116] <= 8'b000000000;
    ROM[117] <= 8'b000000000;
    ROM[118] <= 8'b000000000;
    ROM[119] <= 8'b000000000;
    ROM[120] <= 8'b000000000;
    ROM[121] <= 8'b000000000;
    ROM[122] <= 8'b000000000;
    ROM[123] <= 8'b000000000;
    ROM[124] <= 8'b000000000;
    ROM[125] <= 8'b000000000;
    ROM[126] <= 8'b000000000;
    ROM[127] <= 8'b000000000;
end 

// write is unused
wire [ADDR_WIDTH-1:0] ADDRA = {ADDR_WIDTH{1'b0}}; 
wire                  WEA   = 1'b0; 
always @ (posedge clock)
begin
    if (WEA) begin
        ROM[ADDRA] <= DATAA;
    end
end
   
assign RADDR = addr_ptr;
   
reg [ROM_SIZE-1:0] RDATA;
always @ (posedge clock)
begin
     RDATA <= ROM[RADDR];
end

assign rdata_out = RDATA;
   
endmodule			      