// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



//
// This ROM stores the field valuemask for GXB RX that needs reconfig across various data rate ranges
//
// field_valmask - particular bit value that needs reconfig, to interpret this, get the field bitmask from the
// other ROM file, let's consider ROM[1], field bitmask=8'h0F, field valmask=8'h0A, this means the value of attribute
// cdr_pll_set_cdr_vco_speed_fix when data rate <1100Mbps is bit[3:0]=1010    
//
module mr_rom_rx_valuemask #(
    parameter NUMBER_OF_MIF_RANGE          = 64,  
    parameter ROM_DEPTH_FOR_EACH_MIF_RANGE = 10,       
    parameter ROM_SIZE                     = 8,   // number of bits per rom address - [7:0] field valmask
    parameter TOTAL_ROM_DEPTH              = 512, 
    parameter ADDR_WIDTH                   = 9    // alt_clogb2(512) 
) (
    input  wire                  clock,
    input  wire [ADDR_WIDTH-1:0] addr_ptr,
    output wire [ROM_SIZE-1:0]   rdata_out
);

reg  [ROM_SIZE-1:0]   ROM [0:TOTAL_ROM_DEPTH-1];
wire [ROM_SIZE-1:0]   DATAA = {ROM_SIZE{1'b0}};
wire [ADDR_WIDTH-1:0] RADDR;

// **** PERL SCIPT TO GENERATE BEGIN **** //   

integer i;

initial begin
    // FIELD_VALMASK
    // ROM OFFSET 0
    ROM[  0] <= 8'h2B; 
    ROM[  1] <= 8'h64;
    ROM[  2] <= 8'h21;
    ROM[  3] <= 8'h00;
    // ROM OFFSET 1
    ROM[  4] <= 8'h22;
    ROM[  5] <= 8'h64;
    ROM[  6] <= 8'h21;
    ROM[  7] <= 8'h00;
    // ROM OFFSET 2
    ROM[  8] <= 8'h23;
    ROM[  9] <= 8'h32;
    ROM[ 10] <= 8'h21;
    ROM[ 11] <= 8'h01;
    // ROM OFFSET 3
    ROM[ 12] <= 8'h23;
    ROM[ 13] <= 8'h32;
    ROM[ 14] <= 8'h21;
    ROM[ 15] <= 8'h02;
    // ROM OFFSET 4
    ROM[ 16] <= 8'h23;
    ROM[ 17] <= 8'h32;
    ROM[ 18] <= 8'h21;
    ROM[ 19] <= 8'h03;
    // ROM OFFSET 5
    ROM[ 20] <= 8'h23;
    ROM[ 21] <= 8'h32;
    ROM[ 22] <= 8'h21;
    ROM[ 23] <= 8'h04;
    // ROM OFFSET 6
    ROM[ 24] <= 8'h23;
    ROM[ 25] <= 8'h32;
    ROM[ 26] <= 8'h21;
    ROM[ 27] <= 8'h05;
    // ROM OFFSET 7
    ROM[ 28] <= 8'h23;
    ROM[ 29] <= 8'h32;
    ROM[ 30] <= 8'h21;
    ROM[ 31] <= 8'h06;
    // ROM OFFSET 8
    ROM[ 32] <= 8'h23;
    ROM[ 33] <= 8'h32;
    ROM[ 34] <= 8'h21;
    ROM[ 35] <= 8'h07;
    // ROM OFFSET 9
    ROM[ 36] <= 8'h23;
    ROM[ 37] <= 8'h32;
    ROM[ 38] <= 8'h21;
    ROM[ 39] <= 8'h08;
    // ROM OFFSET 10
    ROM[ 40] <= 8'h23;
    ROM[ 41] <= 8'h32;
    ROM[ 42] <= 8'h21;
    ROM[ 43] <= 8'h09;
    // ROM OFFSET 11
    ROM[ 44] <= 8'h23;
    ROM[ 45] <= 8'h32;
    ROM[ 46] <= 8'h21;
    ROM[ 47] <= 8'h0A;
    // ROM OFFSET 12
    ROM[ 48] <= 8'h23;
    ROM[ 49] <= 8'h32;
    ROM[ 50] <= 8'h21;
    ROM[ 51] <= 8'h0B;
    // ROM OFFSET 13
    ROM[ 52] <= 8'h23;
    ROM[ 53] <= 8'h32;
    ROM[ 54] <= 8'h21;
    ROM[ 55] <= 8'h0C;
    // ROM OFFSET 14
    ROM[ 56] <= 8'h23;
    ROM[ 57] <= 8'h32;
    ROM[ 58] <= 8'h21;
    ROM[ 59] <= 8'h0D;
    // ROM OFFSET 15
    ROM[ 60] <= 8'h23;
    ROM[ 61] <= 8'h32;
    ROM[ 62] <= 8'h21;
    ROM[ 63] <= 8'h0E;
    // ROM OFFSET 16
    ROM[ 64] <= 8'h23; 
    ROM[65] <= 8'h32;
    ROM[66] <= 8'h21;
    ROM[67] <= 8'h0F;
    // ROM OFFSET 17
    ROM[68] <= 8'h23;
    ROM[69] <= 8'h32;
    ROM[70] <= 8'h21;
    ROM[71] <= 8'h10;
    // ROM OFFSET 18
    ROM[72] <= 8'h23;
    ROM[73] <= 8'h32;
    ROM[74] <= 8'h21;
    ROM[75] <= 8'h11;
    // ROM OFFSET 19
    ROM[76] <= 8'h23;
    ROM[77] <= 8'h32;
    ROM[78] <= 8'h21;
    ROM[79] <= 8'h12;
    // ROM OFFSET 20
    ROM[80] <= 8'h1A;
    ROM[81] <= 8'h32;
    ROM[82] <= 8'h21;
    ROM[83] <= 8'h13;
    // ROM OFFSET 21
    ROM[84] <= 8'h33;
    ROM[85] <= 8'h28;
    ROM[86] <= 8'h21;
    ROM[87] <= 8'h14;
    // ROM OFFSET 22
    ROM[88] <= 8'h33;
    ROM[89] <= 8'h28;
    ROM[90] <= 8'h21;
    ROM[91] <= 8'h15;
    // ROM OFFSET 23
    ROM[92] <= 8'h33;
    ROM[93] <= 8'h28;
    ROM[94] <= 8'h21;
    ROM[95] <= 8'h16;
    // ROM OFFSET 24
    ROM[96] <= 8'h33;
    ROM[97] <= 8'h28;
    ROM[98] <= 8'h21;
    ROM[99] <= 8'h17;
    // ROM OFFSET 25
    ROM[100] <= 8'h33;
    ROM[101] <= 8'h28;
    ROM[102] <= 8'h21;
    ROM[103] <= 8'h18;
    // ROM OFFSET 26
    ROM[104] <= 8'h33;
    ROM[105] <= 8'h28;
    ROM[106] <= 8'h21;
    ROM[107] <= 8'h19;
    // ROM OFFSET 27
    ROM[108] <= 8'h33;
    ROM[109] <= 8'h28;
    ROM[110] <= 8'h21;
    ROM[111] <= 8'h1A;
    // ROM OFFSET 28
    ROM[112] <= 8'h33; 
    ROM[113] <= 8'h28;
    ROM[114] <= 8'h21;
    ROM[115] <= 8'h1B;
    // ROM OFFSET 29
    ROM[116] <= 8'h33;
    ROM[117] <= 8'h28;
    ROM[118] <= 8'h21;
    ROM[119] <= 8'h1C;
    // ROM OFFSET 30
    ROM[120] <= 8'h2A;
    ROM[121] <= 8'h28;
    ROM[122] <= 8'h21;
    ROM[123] <= 8'h1D;
    // ROM OFFSET 31
    ROM[124] <= 8'h2A;
    ROM[125] <= 8'h28;
    ROM[126] <= 8'h21;
    ROM[127] <= 8'h1E;
    // ROM OFFSET 32
    ROM[128] <= 8'h2A;
    ROM[129] <= 8'h28;
    ROM[130] <= 8'h21;
    ROM[131] <= 8'h1F;
    // ROM OFFSET 33
    ROM[132] <= 8'h2B; 
    ROM[133] <= 8'h14;
    ROM[134] <= 8'h21;
    ROM[135] <= 8'h1F;
    // ROM OFFSET 34
    ROM[136] <= 8'h2B;
    ROM[137] <= 8'h14;
    ROM[138] <= 8'h21;
    ROM[139] <= 8'h20;
    // ROM OFFSET 35
    ROM[140] <= 8'h2B;
    ROM[141] <= 8'h14;
    ROM[142] <= 8'h21;
    ROM[143] <= 8'h21;
    // ROM OFFSET 36
    ROM[144] <= 8'h2B;
    ROM[145] <= 8'h14;
    ROM[146] <= 8'h21;
    ROM[147] <= 8'h22;
    // ROM OFFSET 37
    ROM[148] <= 8'h2B;
    ROM[149] <= 8'h14;
    ROM[150] <= 8'h21;
    ROM[151] <= 8'h23;
    // ROM OFFSET 38
    ROM[152] <= 8'h2B; 
    ROM[153] <= 8'h14;
    ROM[154] <= 8'h21;
    ROM[155] <= 8'h24;
    // ROM OFFSET 39
    ROM[156] <= 8'h2B;
    ROM[157] <= 8'h14;
    ROM[158] <= 8'h21;
    ROM[159] <= 8'h25;
    // ROM OFFSET 40
    ROM[160] <= 8'h2B;
    ROM[161] <= 8'h14;
    ROM[162] <= 8'h21;
    ROM[163] <= 8'h26;
    // ROM OFFSET 41
    ROM[164] <= 8'h2B;
    ROM[165] <= 8'h14;
    ROM[166] <= 8'h21;
    ROM[167] <= 8'h27;
    // ROM OFFSET 42
    ROM[168] <= 8'h2B;
    ROM[169] <= 8'h14;
    ROM[170] <= 8'h21;
    ROM[171] <= 8'h28;
    // ROM OFFSET 43
    ROM[172] <= 8'h2B;
    ROM[173] <= 8'h14;
    ROM[174] <= 8'h21;
    ROM[175] <= 8'h29;
    // ROM OFFSET 44
    ROM[176] <= 8'h2B;
    ROM[177] <= 8'h14;
    ROM[178] <= 8'h21;
    ROM[179] <= 8'h2A;
    // ROM OFFSET 45
    ROM[180] <= 8'h2B;
    ROM[181] <= 8'h14;
    ROM[182] <= 8'h21;
    ROM[183] <= 8'h2B;
    // ROM OFFSET 46
    ROM[184] <= 8'h2B;
    ROM[185] <= 8'h14;
    ROM[186] <= 8'h21;
    ROM[187] <= 8'h2C;
    // ROM OFFSET 47
    ROM[188] <= 8'h2B; 
    ROM[189] <= 8'h14;
    ROM[190] <= 8'h21;
    ROM[191] <= 8'h2D;
    // ROM OFFSET 48
    ROM[192] <= 8'h2B;
    ROM[193] <= 8'h14;
    ROM[194] <= 8'h21;
    ROM[195] <= 8'h2E;
    // ROM OFFSET 49
    ROM[196] <= 8'h2B;
    ROM[197] <= 8'h14;
    ROM[198] <= 8'h21;
    ROM[199] <= 8'h2F;
    // ROM OFFSET 50
    ROM[200] <= 8'h2B;
    ROM[201] <= 8'h14;
    ROM[202] <= 8'h21;
    ROM[203] <= 8'h30;
    // ROM OFFSET 51
    ROM[204] <= 8'h2B;
    ROM[205] <= 8'h14;
    ROM[206] <= 8'h21;
    ROM[207] <= 8'h31;
    // ROM OFFSET 52
    ROM[208] <= 8'h2B;
    ROM[209] <= 8'h14;
    ROM[210] <= 8'h21;
    ROM[211] <= 8'h32;
    // ROM OFFSET 53
    ROM[212] <= 8'h2B;
    ROM[213] <= 8'h14;
    ROM[214] <= 8'h21;
    ROM[215] <= 8'h33;
    // ROM OFFSET 54
    ROM[216] <= 8'h2B;
    ROM[217] <= 8'h14;
    ROM[218] <= 8'h21;
    ROM[219] <= 8'h34;
    // ROM OFFSET 55
    ROM[220] <= 8'h2B;
    ROM[221] <= 8'h14;
    ROM[222] <= 8'h21;
    ROM[223] <= 8'h35;
    // ROM OFFSET 56
    ROM[224] <= 8'h2B;
    ROM[225] <= 8'h14;
    ROM[226] <= 8'h21;
    ROM[227] <= 8'h36;
    // ROM OFFSET 57
    ROM[228] <= 8'h2B;
    ROM[229] <= 8'h14;
    ROM[230] <= 8'h21;
    ROM[231] <= 8'h37;
    // ROM OFFSET 58
    ROM[232] <= 8'h2B;
    ROM[233] <= 8'h14;
    ROM[234] <= 8'h21;
    ROM[235] <= 8'h38;
    // ROM OFFSET 59
    ROM[236] <= 8'h2B;
    ROM[237] <= 8'h14;
    ROM[238] <= 8'h21;
    ROM[239] <= 8'h39;
    // ROM OFFSET 60
    ROM[240] <= 8'h2B;
    ROM[241] <= 8'h14;
    ROM[242] <= 8'h21;
    ROM[243] <= 8'h3A;
    // ROM OFFSET 61
    ROM[244] <= 8'h2B;
    ROM[245] <= 8'h14;
    ROM[246] <= 8'h21;
    ROM[247] <= 8'h3B;
    // ROM OFFSET 62
    ROM[248] <= 8'h2B;
    ROM[249] <= 8'h14;
    ROM[250] <= 8'h21;
    ROM[251] <= 8'h3C;
    // ROM OFFSET 63
    ROM[252] <= 8'h2B;
    ROM[253] <= 8'h14;
    ROM[254] <= 8'h21;
    ROM[255] <= 8'h3D;
    // ROM OFFSET 64
    ROM[256] <= 8'h2B;
    ROM[257] <= 8'h14;
    ROM[258] <= 8'h21;
    ROM[259] <= 8'h3E;
    // ROM OFFSET 65
    ROM[260] <= 8'h2B;
    ROM[261] <= 8'h14;
    ROM[262] <= 8'h21;
    ROM[263] <= 8'h3F;
    // ROM OFFSET 66
    ROM[264] <= 8'h2B;
    ROM[265] <= 8'h14;
    ROM[266] <= 8'h21;
    ROM[267] <= 8'h40;
    // ROM OFFSET 67
    ROM[268] <= 8'h2B;
    ROM[269] <= 8'h14;
    ROM[270] <= 8'h21;
    ROM[271] <= 8'h41;
    // ROM OFFSET 68
    ROM[272] <= 8'h2B;
    ROM[273] <= 8'h14;
    ROM[274] <= 8'h21;
    ROM[275] <= 8'h42;
    // ROM OFFSET 69
    ROM[276] <= 8'h2B;
    ROM[277] <= 8'h14;
    ROM[278] <= 8'h21;
    ROM[279] <= 8'h43;
    // ROM OFFSET 70
    ROM[280] <= 8'h2B;
    ROM[281] <= 8'h14;
    ROM[282] <= 8'h21; 
    ROM[283] <= 8'h44;
    // ROM OFFSET 71
    ROM[284] <= 8'h2B;
    ROM[285] <= 8'h14;
    ROM[286] <= 8'h21;
    ROM[287] <= 8'h45;
    // ROM OFFSET 72
    ROM[288] <= 8'h22;
    ROM[289] <= 8'h14;
    ROM[290] <= 8'h21;
    ROM[291] <= 8'h46;
    // ROM OFFSET 73
    ROM[292] <= 8'h22;
    ROM[293] <= 8'h14;
    ROM[294] <= 8'h21;
    ROM[295] <= 8'h47;
    // ROM OFFSET 74
    ROM[296] <= 8'h22;
    ROM[297] <= 8'h14;
    ROM[298] <= 8'h21;
    ROM[299] <= 8'h48;
    // ROM OFFSET 75
    ROM[300] <= 8'h23;
    ROM[301] <= 8'h0A;
    ROM[302] <= 8'h21;
    ROM[303] <= 8'h48;
    // ROM OFFSET 76
    ROM[304] <= 8'h23;
    ROM[305] <= 8'h0A;
    ROM[306] <= 8'h21;
    ROM[307] <= 8'h49;
    // ROM OFFSET 77
    ROM[308] <= 8'h23;
    ROM[309] <= 8'h0A;
    ROM[310] <= 8'h21;
    ROM[311] <= 8'h4A;
    // ROM OFFSET 78
    ROM[312] <= 8'h23;
    ROM[313] <= 8'h0A;
    ROM[314] <= 8'h21;
    ROM[315] <= 8'h4B;
    // ROM OFFSET 79
    ROM[316] <= 8'h23;
    ROM[317] <= 8'h0A;
    ROM[318] <= 8'h21;
    ROM[319] <= 8'h4C;
    // ROM OFFSET 80
    ROM[320] <= 8'h23;
    ROM[321] <= 8'h0A;
    ROM[322] <= 8'h21;
    ROM[323] <= 8'h4D;
    // ROM OFFSET 81
    ROM[324] <= 8'h23;
    ROM[325] <= 8'h0A;
    ROM[326] <= 8'h21;
    ROM[327] <= 8'h4E;
    // ROM OFFSET 82
    ROM[328] <= 8'h23;
    ROM[329] <= 8'h0A;
    ROM[330] <= 8'h21;
    ROM[331] <= 8'h4F;
    // ROM OFFSET 83
    ROM[332] <= 8'h23;
    ROM[333] <= 8'h0A;
    ROM[334] <= 8'h21;
    ROM[335] <= 8'h50;
    // ROM OFFSET 84
    ROM[336] <= 8'h23;
    ROM[337] <= 8'h0A;
    ROM[338] <= 8'h21;
    ROM[339] <= 8'h51;
    // ROM OFFSET 85
    ROM[340] <= 8'h23;
    ROM[341] <= 8'h0A;
    ROM[342] <= 8'h21;
    ROM[343] <= 8'h52;
    // ROM OFFSET 86
    ROM[344] <= 8'h23;
    ROM[345] <= 8'h0A;
    ROM[346] <= 8'h21;
    ROM[347] <= 8'h53;
    // ROM OFFSET 87
    ROM[348] <= 8'h23;
    ROM[349] <= 8'h0A;
    ROM[350] <= 8'h21;
    ROM[351] <= 8'h54;
    // ROM OFFSET 88
    ROM[352] <= 8'h23;
    ROM[353] <= 8'h0A;
    ROM[354] <= 8'h21;
    ROM[355] <= 8'h55;
    // ROM OFFSET 89
    ROM[356] <= 8'h23;
    ROM[357] <= 8'h0A;
    ROM[358] <= 8'h21;
    ROM[359] <= 8'h56;
    // ROM OFFSET 90
    ROM[360] <= 8'h23;
    ROM[361] <= 8'h0A;
    ROM[362] <= 8'h21;
    ROM[363] <= 8'h57;
    // ROM OFFSET 91
    ROM[364] <= 8'h23;
    ROM[365] <= 8'h0A;
    ROM[366] <= 8'h21;
    ROM[367] <= 8'h58;
    // ROM OFFSET 92
    ROM[368] <= 8'h23;
    ROM[369] <= 8'h28;
    ROM[370] <= 8'h21;
    ROM[371] <= 8'h0E;
    // ROM OFFSET 93
    ROM[372] <= 8'h23;
    ROM[373] <= 8'h28;
    ROM[374] <= 8'h21;
    ROM[375] <= 8'h0F;
    // ROM OFFSET 94
    ROM[376] <= 8'h23;
    ROM[377] <= 8'h28;
    ROM[378] <= 8'h21;
    ROM[379] <= 8'h10;
    // ROM OFFSET 95
    ROM[380] <= 8'h23;
    ROM[381] <= 8'h28;
    ROM[382] <= 8'h21;
    ROM[383] <= 8'h11;
    // ROM OFFSET 96
    ROM[384] <= 8'h23;
    ROM[385] <= 8'h28;
    ROM[386] <= 8'h21;
    ROM[387] <= 8'h12;
    // ROM OFFSET 97
    ROM[388] <= 8'h23;
    ROM[389] <= 8'h28;
    ROM[390] <= 8'h21;
    ROM[391] <= 8'h13;
    // ROM OFFSET 98
    ROM[392] <= 8'h23;
    ROM[393] <= 8'h28;
    ROM[394] <= 8'h21;
    ROM[395] <= 8'h14;
    // ROM OFFSET 99
    ROM[396] <= 8'h23;
    ROM[397] <= 8'h28;
    ROM[398] <= 8'h21;
    ROM[399] <= 8'h15;
    // ROM OFFSET 100
    ROM[400] <= 8'h23;
    ROM[401] <= 8'h28;
    ROM[402] <= 8'h21;
    ROM[403] <= 8'h16;
    // ROM OFFSET 101
    ROM[404] <= 8'h23;
    ROM[405] <= 8'h28;
    ROM[406] <= 8'h21;
    ROM[407] <= 8'h17;
    // ROM OFFSET 102
    ROM[408] <= 8'h23;
    ROM[409] <= 8'h28;
    ROM[410] <= 8'h21;
    ROM[411] <= 8'h18;
    // ROM OFFSET 103
    ROM[412] <= 8'h23;
    ROM[413] <= 8'h28;
    ROM[414] <= 8'h21;
    ROM[415] <= 8'h19;
    // ROM OFFSET 104
    ROM[416] <= 8'h23;
    ROM[417] <= 8'h28;
    ROM[418] <= 8'h21;
    ROM[419] <= 8'h1A;
    // ROM OFFSET 105
    ROM[420] <= 8'h23;
    ROM[421] <= 8'h28;
    ROM[422] <= 8'h21;
    ROM[423] <= 8'h1B;
    // ROM OFFSET 106
    ROM[424] <= 8'h23;
    ROM[425] <= 8'h28;
    ROM[426] <= 8'h21;
    ROM[427] <= 8'h1C;
    // ROM OFFSET 107
    ROM[428] <= 8'h1A;
    ROM[429] <= 8'h28;
    ROM[430] <= 8'h21;
    ROM[431] <= 8'h1D;
    // ROM OFFSET 108
    ROM[432] <= 8'h1A;
    ROM[433] <= 8'h28;
    ROM[434] <= 8'h21;
    ROM[435] <= 8'h1E;
    // ROM OFFSET 109
    ROM[436] <= 8'h1A;
    ROM[437] <= 8'h28;
    ROM[438] <= 8'h21;
    ROM[439] <= 8'h1F;
    // ROM OFFSET 110
    ROM[440] <= 8'h1B;
    ROM[441] <= 8'h14;
    ROM[442] <= 8'h21;
    ROM[443] <= 8'h1F;
    // ROM OFFSET 111
    ROM[444] <= 8'h1B;
    ROM[445] <= 8'h14;
    ROM[446] <= 8'h21;
    ROM[447] <= 8'h20;
    // ROM OFFSET 112
    ROM[448] <= 8'h1B;
    ROM[449] <= 8'h14;
    ROM[450] <= 8'h21;
    ROM[451] <= 8'h21;
    // ROM OFFSET 113
    ROM[452] <= 8'h1B;
    ROM[453] <= 8'h14;
    ROM[454] <= 8'h21;
    ROM[455] <= 8'h22;
    // ROM OFFSET 114
    ROM[456] <= 8'h1B;
    ROM[457] <= 8'h14;
    ROM[458] <= 8'h21;
    ROM[459] <= 8'h23;
    // ROM OFFSET 115
    ROM[460] <= 8'h1B;
    ROM[461] <= 8'h14;
    ROM[462] <= 8'h21;
    ROM[463] <= 8'h24;
    // ROM OFFSET 116
    ROM[464] <= 8'h1B;
    ROM[465] <= 8'h14;
    ROM[466] <= 8'h21;
    ROM[467] <= 8'h25;
    // ROM OFFSET 117
    ROM[468] <= 8'h1B;
    ROM[469] <= 8'h14;
    ROM[470] <= 8'h21;
    ROM[471] <= 8'h26;
    // ROM OFFSET 118
    ROM[472] <= 8'h1B;
    ROM[473] <= 8'h14;
    ROM[474] <= 8'h21;
    ROM[475] <= 8'h27;
    // ROM OFFSET 119
    ROM[476] <= 8'h1B;
    ROM[477] <= 8'h14;
    ROM[478] <= 8'h21;
    ROM[479] <= 8'h28;
    
    //Reserved
    for (i=480; i<512; i=i+1) begin
      ROM[i] <= 8'h00;
    end

end
// **** PERL SCIPT TO GENERATE END ****** //

// write is unused
wire [ADDR_WIDTH-1:0] ADDRA = {ADDR_WIDTH{1'b0}}; 
wire                  WEA   = 1'b0; 
always @ (posedge clock)
begin
    if (WEA) begin
        ROM[ADDRA] <= DATAA;
    end
end
   
assign RADDR = addr_ptr;
   
reg [ROM_SIZE-1:0] RDATA;
always @ (posedge clock)
begin
     RDATA <= ROM[RADDR];
end

assign rdata_out = RDATA;

endmodule
