// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


module rxtx_link #(
    parameter SYMBOLS_PER_CLOCK = 4
) (

   input  reset,
   input  mgmt_clk,
   input  i2c_clk,
   input  hdmi_tx_ls_clk,
   //output hdmi_tx_ls_clk_reset_sync,
   input  hdmi_rx_ls_clk,
   input  hdmi_tx_vid_clk,
   input  hdmi_rx_vid_clk,
   input  hdmi_rx_locked,
   input  user_pb_2,

   input  [SYMBOLS_PER_CLOCK-1:0] hdmi_rx_de,
   input  [SYMBOLS_PER_CLOCK-1:0] hdmi_rx_hsync,
   input  [SYMBOLS_PER_CLOCK-1:0] hdmi_rx_vsync,
   input  [SYMBOLS_PER_CLOCK*48-1:0] hdmi_rx_data,
   input  [4:0] rx_audio_format,
   input  [164:0] rx_audio_metadata,
   input  [47:0] rx_audio_info_ai,
   input  [19:0] rx_audio_CTS,
   input  [19:0] rx_audio_N,
   input  rx_audio_de,
   input  [255:0] rx_audio_data,
   input  [5:0] rx_gcp,
   input  [111:0] rx_info_avi,
   input  [60:0] rx_info_vsi,
   input  rx_aux_eop,
   input  rx_aux_sop,
   input  rx_aux_valid,
   input  [71:0] rx_aux_data,

   output reg [1*SYMBOLS_PER_CLOCK-1:0] hdmi_tx_de,
   output reg [1*SYMBOLS_PER_CLOCK-1:0] hdmi_tx_hsync,
   output reg [1*SYMBOLS_PER_CLOCK-1:0] hdmi_tx_vsync,
   output reg [48*SYMBOLS_PER_CLOCK-1:0] hdmi_tx_data,
   output reg [4:0] tx_audio_format,
   output reg [164:0] tx_audio_metadata,
   output reg [47:0] tx_audio_info_ai,
   output reg [19:0] tx_audio_CTS,
   output reg [19:0] tx_audio_N,
   output reg tx_audio_de,
   output reg [255:0] tx_audio_data,
   output reg [5:0] tx_gcp,
   output reg [111:0] tx_info_avi,
   output reg [60:0] tx_info_vsi,
   output reg tx_aux_eop,
   output reg tx_aux_sop,
   output reg tx_aux_valid,
   output reg [71:0] tx_aux_data,
   input  tx_aux_ready,
  
   //output [1:0] color_depth_sync,
   //output hdmi_rx_locked_sync,
   input  sys_init,
   input  wd_reset
   //output rx_mgmt_clk_reset
   

);

wire reset_sync_txlsclk;
wire reset_sync_txvidclk;
wire reset_sync_rxlsclk;
wire reset_sync_rxvidclk;

//
// HDMI video/audio/aux/infoframes bypass fifos
//
reg vid_bp_ff_wrreq;
reg  vid_bp_ff_rdreq;
wire vid_bp_ff_full;
wire vid_bp_ff_mt;
wire vid_bp_ff_almost_full;
wire vid_bp_ff_almost_mt;
wire [4:0] vid_bp_ff_wrusedw;
wire [4:0] vid_bp_ff_rdusedw;
wire [1*SYMBOLS_PER_CLOCK-1:0] hdmi_tx_de_w;
wire [1*SYMBOLS_PER_CLOCK-1:0] hdmi_tx_hsync_w;
wire [1*SYMBOLS_PER_CLOCK-1:0] hdmi_tx_vsync_w;
wire [48*SYMBOLS_PER_CLOCK-1:0] hdmi_tx_data_w;

assign vid_bp_ff_almost_full = vid_bp_ff_wrusedw >= 29;
assign vid_bp_ff_almost_mt = vid_bp_ff_rdusedw <= 2;
//assign vid_bp_ff_wrreq = ~vid_bp_ff_full;
// assign vid_bp_ff_rdreq = ~vid_bp_ff_mt;

always @ (posedge hdmi_tx_vid_clk or posedge reset_sync_txvidclk)
begin
    if (reset_sync_txvidclk) begin
        vid_bp_ff_rdreq <= 1'b0;
    end else begin
        vid_bp_ff_rdreq <= ~vid_bp_ff_almost_mt;
    end
end

always @ (posedge hdmi_rx_vid_clk or posedge reset_sync_rxvidclk)
begin
    if (reset_sync_rxvidclk) begin
        vid_bp_ff_wrreq <= 1'b0;
    end else begin
        vid_bp_ff_wrreq <= ~vid_bp_ff_almost_full;
    end
end

dcfifo_inst #(.WIDTH(102)) u_vid_bypass_fifo (
   /* I */ .rdclk (hdmi_tx_vid_clk),
   /* I */ .wrclk (hdmi_rx_vid_clk),
   /* I */ .wrreq (vid_bp_ff_wrreq),
   /* I */ .aclr (reset | ~hdmi_rx_locked),
   /* I */ .data ({hdmi_rx_de, hdmi_rx_hsync, hdmi_rx_vsync, hdmi_rx_data}),
   /* I */ .rdreq (vid_bp_ff_rdreq),
   /* O */ .wrfull (vid_bp_ff_full),
   /* O */ .q ({hdmi_tx_de_w, hdmi_tx_hsync_w, hdmi_tx_vsync_w, hdmi_tx_data_w}),
   /* O */ .rdempty (vid_bp_ff_mt),
   /* O */ .wrusedw (vid_bp_ff_wrusedw),
   /* O */ .rdusedw (vid_bp_ff_rdusedw));

reg aud_bp_ff_wrreq;
reg  aud_bp_ff_rdreq;
wire aud_bp_ff_full;
wire aud_bp_ff_mt;
wire aud_bp_ff_almost_full;
wire aud_bp_ff_almost_mt;
wire [4:0] aud_bp_ff_wrusedw;
wire [4:0] aud_bp_ff_rdusedw;
wire [4:0] tx_audio_format_w;
wire [164:0] tx_audio_metadata_w;
wire [47:0] tx_audio_info_ai_w;
wire [19:0] tx_audio_CTS_w;
wire [19:0] tx_audio_N_w;
wire tx_audio_de_w;
wire [255:0] tx_audio_data_w;

assign aud_bp_ff_almost_full = aud_bp_ff_wrusedw >= 29;
assign aud_bp_ff_almost_mt = aud_bp_ff_rdusedw <= 2;
//assign aud_bp_ff_wrreq = ~aud_bp_ff_full;
// assign aud_bp_ff_rdreq = ~aud_bp_ff_mt;

always @ (posedge hdmi_tx_ls_clk or posedge reset_sync_txlsclk)
begin
    if (reset_sync_txlsclk) begin
        aud_bp_ff_rdreq <= 1'b0;
    end else begin
        aud_bp_ff_rdreq <= ~aud_bp_ff_almost_mt;
    end
end

always @ (posedge hdmi_rx_ls_clk or posedge reset_sync_rxlsclk)
begin
    if (reset_sync_rxlsclk) begin
        aud_bp_ff_wrreq <= 1'b0;
    end else begin
        aud_bp_ff_wrreq <= ~aud_bp_ff_almost_full;
    end
end

dcfifo_inst #( .WIDTH(5 + 165+ 48 + 20 + 20 + 1 + 256)) u_aud_bypass_fifo (
   /* I */ .rdclk (hdmi_tx_ls_clk),
   /* I */ .wrclk (hdmi_rx_ls_clk),
   /* I */ .wrreq (aud_bp_ff_wrreq),
   /* I */ .aclr (reset | ~hdmi_rx_locked),
   /* I */ .data ({rx_audio_format, rx_audio_metadata, rx_audio_info_ai, rx_audio_CTS, rx_audio_N, rx_audio_de, rx_audio_data}),
   /* I */ .rdreq (aud_bp_ff_rdreq),
   /* O */ .wrfull (aud_bp_ff_full),
   /* O */ .q ({tx_audio_format_w, tx_audio_metadata_w, tx_audio_info_ai_w, tx_audio_CTS_w, tx_audio_N_w, tx_audio_de_w, tx_audio_data_w}),
   /* O */ .rdempty (aud_bp_ff_mt),
   /* O */ .wrusedw (aud_bp_ff_wrusedw),
   /* O */ .rdusedw (aud_bp_ff_rdusedw));

reg if_bp_ff_wrreq;
reg  if_bp_ff_rdreq;
wire if_bp_ff_full;
wire if_bp_ff_mt;
wire if_bp_ff_almost_full;
wire if_bp_ff_almost_mt;
wire [4:0] if_bp_ff_wrusedw;
wire [4:0] if_bp_ff_rdusedw;
wire [5:0] tx_gcp_w;
wire [111:0] tx_info_avi_w;
wire [60:0] tx_info_vsi_w;

assign if_bp_ff_almost_full = if_bp_ff_wrusedw >= 29;
assign if_bp_ff_almost_mt = if_bp_ff_rdusedw <= 2;
//assign if_bp_ff_wrreq = ~if_bp_ff_full;
// assign if_bp_ff_rdreq = ~if_bp_ff_mt;

always @ (posedge hdmi_tx_ls_clk or posedge reset_sync_txlsclk)
begin
    if (reset_sync_txlsclk) begin
        if_bp_ff_rdreq <= 1'b0;
    end else begin
        if_bp_ff_rdreq <= ~if_bp_ff_almost_mt;
    end
end

always @ (posedge hdmi_rx_ls_clk or posedge reset_sync_rxlsclk)
begin
    if (reset_sync_rxlsclk) begin
        if_bp_ff_wrreq <= 1'b0;
    end else begin
        if_bp_ff_wrreq <= ~if_bp_ff_almost_full;
    end
end

dcfifo_inst #(.WIDTH(179)) u_if_passthru_fifo (
   /* I */ .rdclk (hdmi_tx_ls_clk),
   /* I */ .wrclk (hdmi_rx_ls_clk),
   /* I */ .wrreq (if_bp_ff_wrreq),
   /* I */ .aclr (reset | ~hdmi_rx_locked),
   /* I */ .data ({rx_gcp, rx_info_avi, rx_info_vsi}),
   /* I */ .rdreq (if_bp_ff_rdreq),
   /* O */ .wrfull (if_bp_ff_full),
   /* O */ .q ({tx_gcp_w, tx_info_avi_w, tx_info_vsi_w}),
   /* O */ .rdempty (if_bp_ff_mt),
   /* O */ .wrusedw (if_bp_ff_wrusedw),
   /* O */ .rdusedw (if_bp_ff_rdusedw));

wire aux_bp_ff_wrreq;
wire aux_bp_ff_rdreq;
wire aux_bp_ff_full;
wire aux_bp_ff_mt;
wire aux_bp_ff_almost_full;
wire aux_bp_ff_almost_mt;
wire [4:0] aux_bp_ff_wrusedw;
wire [4:0] aux_bp_ff_rdusedw;
wire [71:0] aux_bp_ff_data;
reg         aux_bp_ff_valid;
reg         aux_bp_ff_valid_r;
wire        aux_bp_ff_sop;
wire        aux_bp_ff_eop;
reg         aux_bp_ff_ready;
reg [71:0]  aux_bp_ff_data_r;
reg         aux_bp_ff_sop_r;
reg         aux_bp_ff_eop_r;

assign aux_bp_ff_wrreq = rx_aux_valid & ~aux_bp_ff_full;
assign aux_bp_ff_rdreq = aux_bp_ff_ready & ~aux_bp_ff_mt;
//assign aux_bp_ff_valid = ~aux_bp_ff_mt;


always @ (posedge hdmi_tx_ls_clk or posedge reset_sync_txlsclk)
begin   
   if (reset_sync_txlsclk) begin
      aux_bp_ff_valid <= 1'b0;
      aux_bp_ff_valid_r <= 1'b0;
   end else begin
        if      (aux_bp_ff_ready & ~aux_bp_ff_mt)    aux_bp_ff_valid <= 1'b1; // 1 clock cycle delay of the read will be the valid for the data
        else if (~aux_bp_ff_ready & aux_bp_ff_valid) aux_bp_ff_valid <= 1'b1; // need to hold the valid until the ready asserted so that data can be read by AV ST MUX
        else                                         aux_bp_ff_valid <= 1'b0;
        
        aux_bp_ff_valid_r <= aux_bp_ff_valid;
   end
end

dcfifo_inst #(.WIDTH(74)) u_aux_passthru_fifo (
   /* I */ .rdclk (hdmi_tx_ls_clk),
   /* I */ .wrclk (hdmi_rx_ls_clk),
   /* I */ .wrreq (aux_bp_ff_wrreq),
   /* I */ .aclr (reset | ~hdmi_rx_locked),
   /* I */ .data ({rx_aux_eop, rx_aux_sop, rx_aux_data}),
   /* I */ .rdreq (aux_bp_ff_rdreq),
   /* O */ .wrfull (aux_bp_ff_full),
   /* O */ .q ({aux_bp_ff_eop, aux_bp_ff_sop, aux_bp_ff_data}),
   /* O */ .rdempty (aux_bp_ff_mt));

always @ (posedge hdmi_tx_ls_clk )
begin
    {aux_bp_ff_eop_r, aux_bp_ff_sop_r, aux_bp_ff_data_r} <= {aux_bp_ff_eop, aux_bp_ff_sop, aux_bp_ff_data};
end
   
//---------------------------------------------------------------------------------------------------------
// High Dynamic Range (HDR) 

wire hdmi_tx_vsync_sync;
wire multiplexer_in_ready;
wire ext_aux_block;
wire tx_aux_eop_w;
wire tx_aux_sop_w;
wire tx_aux_valid_w;
wire [71:0] tx_aux_data_w;

wire tx_aux_eop_pipe;
wire tx_aux_sop_pipe;
wire tx_aux_valid_pipe;
wire tx_aux_ready_pipe;
wire [71:0] tx_aux_data_pipe;

clock_crosser #(.W(1)) hdmi_tx_vsync_cc (.in(hdmi_tx_vsync[0]), .out(hdmi_tx_vsync_sync), .in_clk(hdmi_tx_vid_clk),.out_clk(hdmi_tx_ls_clk),.in_reset(1'b0),.out_reset(1'b0));

altera_hdmi_aux_hdr altera_hdmi_aux_hdr_inst (
  .clk(hdmi_tx_ls_clk),
  .reset(reset_sync_txlsclk),
  .hdmi_tx_vsync(hdmi_tx_vsync_sync),
  .multiplexer_in1_data(aux_bp_ff_data_r),
  .multiplexer_in1_valid(aux_bp_ff_valid_r & ~ext_aux_block),
  .multiplexer_in1_ready(multiplexer_in_ready),
  .multiplexer_in1_startofpacket(aux_bp_ff_sop_r),
  .multiplexer_in1_endofpacket(aux_bp_ff_eop_r),
  .multiplexer_out_data(tx_aux_data_pipe),
  .multiplexer_out_valid(tx_aux_valid_pipe),
  .multiplexer_out_ready(tx_aux_ready_pipe),
  .multiplexer_out_startofpacket(tx_aux_sop_pipe),
  .multiplexer_out_endofpacket(tx_aux_eop_pipe),
  .multiplexer_out_channel()
  
);

bitec_hdmi_tx_aux_encoder_pipeline hdr_pipe (
    .clk        (hdmi_tx_ls_clk),
    .reset      (reset_sync_txlsclk),
    .in_ready   (tx_aux_ready_pipe),
    .in_valid   (tx_aux_valid_pipe),
    .in_data    ({tx_aux_eop_pipe, tx_aux_sop_pipe, tx_aux_data_pipe}),
    .out_ready  (tx_aux_ready),
    .out_valid  (tx_aux_valid_w),
    .out_data   ({tx_aux_eop_w, tx_aux_sop_w, tx_aux_data_w})
);
   
//---------------------------------------------------------------------------------------------------------
//
// Example of external filtering of audio sample, timestamp on the auxiliary data port which is loopback from the sink.
// (Workaround) Filter the audio infoframe on the auxiliary data port as well when msb of audio_info_ai is 0.
//

wire block_ext_aud_related_packet;
wire block_ext_hdr_packet;
wire block_ext_aud_sample;
wire block_ext_aud_timestamp;
wire block_ext_aud_infoframe;
wire block_ext_hdr_infoframe;
wire block_ext_aux_packet;
wire ext_aud_sample;
wire ext_aud_timestamp;
wire ext_aud_infoframe;
wire ext_hdr_infoframe;
reg do_ext_aux_block;

assign block_ext_aud_sample = 1'b1;
assign block_ext_aud_timestamp = 1'b1;
assign block_ext_aud_infoframe = ~user_pb_2;
assign block_ext_hdr_infoframe = 1'b1;
assign ext_aud_sample = aux_bp_ff_sop_r & (aux_bp_ff_data_r[7:0] == 8'h02) & aux_bp_ff_valid_r;
assign ext_aud_timestamp = aux_bp_ff_sop_r & (aux_bp_ff_data_r[7:0] == 8'h01) & aux_bp_ff_valid_r;
assign ext_aud_infoframe = aux_bp_ff_sop_r & (aux_bp_ff_data_r[7:0] == 8'h84) & aux_bp_ff_valid_r;
assign ext_hdr_infoframe = aux_bp_ff_sop_r & (aux_bp_ff_data_r[7:0] == 8'h87) & aux_bp_ff_valid_r;
assign block_ext_aud_related_packet = (block_ext_aud_sample & ext_aud_sample) | 
                                      (block_ext_aud_timestamp & ext_aud_timestamp) |
                                      (block_ext_aud_infoframe & ext_aud_infoframe);
assign block_ext_hdr_packet = block_ext_hdr_infoframe & ext_hdr_infoframe;

assign block_ext_aux_packet = block_ext_aud_related_packet | block_ext_hdr_packet;

always @ (posedge hdmi_tx_ls_clk or posedge reset_sync_txlsclk)
begin   
   if (reset_sync_txlsclk) begin
      do_ext_aux_block <= 1'b0;
      aux_bp_ff_ready <= 1'b0;
   end else begin
      do_ext_aux_block <= do_ext_aux_block ? ~(aux_bp_ff_eop_r & aux_bp_ff_valid_r) : block_ext_aux_packet;
      aux_bp_ff_ready <= multiplexer_in_ready | ext_aux_block;
   end
end
    
assign ext_aux_block = block_ext_aux_packet | do_ext_aux_block; 
// assign aux_bp_ff_ready = multiplexer_in_ready | ext_aux_block;


// Flop when outputing the signal
always @ (posedge hdmi_tx_ls_clk or posedge reset_sync_txlsclk)
begin
   if (reset_sync_txlsclk)
      tx_aux_valid <= 1'b0;
   else
      tx_aux_valid <= tx_aux_valid_w;
end

always @ (posedge hdmi_tx_ls_clk)
begin
      {tx_aux_data, tx_aux_sop, tx_aux_eop} <= {tx_aux_data_w, tx_aux_sop_w, tx_aux_eop_w};
      {tx_gcp, tx_info_avi, tx_info_vsi} <= {tx_gcp_w, tx_info_avi_w, tx_info_vsi_w};
      {tx_audio_format, tx_audio_metadata, tx_audio_info_ai, tx_audio_CTS, tx_audio_N, tx_audio_de, tx_audio_data} <= {tx_audio_format_w, tx_audio_metadata_w, tx_audio_info_ai_w, tx_audio_CTS_w, tx_audio_N_w, tx_audio_de_w, tx_audio_data_w};
end

always @ (posedge hdmi_tx_vid_clk)
begin
      {hdmi_tx_de, hdmi_tx_hsync, hdmi_tx_vsync, hdmi_tx_data} <= {hdmi_tx_de_w, hdmi_tx_hsync_w, hdmi_tx_vsync_w, hdmi_tx_data_w};
end

altera_reset_controller #(
   .NUM_RESET_INPUTS          (1),
   .SYNC_DEPTH                (3),
   .RESET_REQ_WAIT_TIME       (1),
   .MIN_RST_ASSERTION_TIME    (3),
   .RESET_REQ_EARLY_DSRT_TIME (1)
) u_reset_sync_txlsclk (
  .reset_in0 (reset),
  .clk (hdmi_tx_ls_clk),
  .reset_out (reset_sync_txlsclk)
);

altera_reset_controller #(
   .NUM_RESET_INPUTS          (1),
   .SYNC_DEPTH                (3),
   .RESET_REQ_WAIT_TIME       (1),
   .MIN_RST_ASSERTION_TIME    (3),
   .RESET_REQ_EARLY_DSRT_TIME (1)
) u_reset_sync_txvidclk (
  .reset_in0 (reset),
  .clk (hdmi_tx_vid_clk),
  .reset_out (reset_sync_txvidclk)
);

altera_reset_controller #(
   .NUM_RESET_INPUTS          (1),
   .SYNC_DEPTH                (3),
   .RESET_REQ_WAIT_TIME       (1),
   .MIN_RST_ASSERTION_TIME    (3),
   .RESET_REQ_EARLY_DSRT_TIME (1)
) u_reset_sync_rxlsclk (
  .reset_in0 (reset),
  .clk (hdmi_rx_ls_clk),
  .reset_out (reset_sync_rxlsclk)
);

altera_reset_controller #(
   .NUM_RESET_INPUTS          (1),
   .SYNC_DEPTH                (3),
   .RESET_REQ_WAIT_TIME       (1),
   .MIN_RST_ASSERTION_TIME    (3),
   .RESET_REQ_EARLY_DSRT_TIME (1)
) u_reset_sync_rxvidclk (
  .reset_in0 (reset),
  .clk (hdmi_rx_vid_clk),
  .reset_out (reset_sync_rxvidclk)
);

endmodule

