// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



//`define DEBUG_EN 1

module hdmi_rx_top #(
   parameter SUPPORT_DEEP_COLOR      = 1,
   parameter SUPPORT_AUXILIARY       = 1,
   parameter SYMBOLS_PER_CLOCK       = 2,
   parameter SUPPORT_AUDIO           = 1,
   parameter POLARITY_INVERSION      = 1,
   parameter BITEC_DAUGHTER_CARD_REV = 6
) (  
   // Clock, reset and PLL locked signals 
   input  wire                                    reset,
   input  wire                                    mgmt_clk,
   input  wire                                    hdmi_clk_in,
   input  wire                                    tmds_clk_in,	
   output wire                                    vid_clk_out,
   output wire                                    ls_clk_out,   
   output wire                                    iopll_locked,
   output wire                                    sys_init,
   // GXB RX signals
   input  wire [2:0]                              rx_serial_data,
   output wire                                    gxb_rx_ready,	
   // HDMI RX signals
   output wire                                    TMDS_Bit_clock_Ratio,
   output wire                                    audio_de,
   output wire [255:0]                            audio_data,
   output wire [47:0]                             audio_info_ai,   
   output wire [19:0]                             audio_N,
   output wire [19:0]                             audio_CTS,
   output wire [164:0]                            audio_metadata,
   output wire [4:0]                              audio_format,
   output wire [71:0]                             aux_pkt_data,
   output wire [6:0]                              aux_pkt_addr,
   output wire                                    aux_pkt_wr,
   output wire [5:0]                              gcp,
   output wire [111:0]                            info_avi,
   output wire [60 :0]                            info_vsi,
   output wire [71:0]                             aux_data,
   output wire                                    aux_sop,
   output wire                                    aux_eop,
   output wire                                    aux_valid,
   output wire                                    aux_error,
   output wire [1:0]                              colordepth_mgmt_sync,
   output wire                                    locked,
   output wire [SYMBOLS_PER_CLOCK*48-1:0]         vid_data,
   output wire [SYMBOLS_PER_CLOCK-1:0]            vid_vsync,
   output wire [SYMBOLS_PER_CLOCK-1:0]            vid_hsync,
   output wire [SYMBOLS_PER_CLOCK-1:0]            vid_de,
   output wire                                    vid_lock,
   input  wire                                    in_5v_power,
   inout  wire                                    hdmi_rx_hpd_n,
   input  wire                                    user_pb_0,
   output wire                                    mode,
   output wire [SYMBOLS_PER_CLOCK*6-1:0]          ctrl,     
   // Reconfig mgmt signals
   output wire [23:0]                             measure,
   output wire                                    measure_valid,
   output wire                                    os, 
   // I2C signals
   inout  wire                                    hdmi_rx_i2c_sda,
   inout  wire                                    hdmi_rx_i2c_scl,
   input  wire                                    i2c_clk,
   //output wire [7:0]                              scdc_i2c_rdata,
   //input  wire                                    scdc_i2c_w,
   //input  wire                                    scdc_i2c_r,
   //input  wire [7:0]                              scdc_i2c_addr,
   //input  wire [7:0]                              scdc_i2c_wdata,

   output wire        reconfig_mgmt_write,
   output wire        reconfig_mgmt_read,
   output wire [12:0] reconfig_mgmt_address,
   output wire [31:0] reconfig_mgmt_writedata,
   input wire  [31:0] reconfig_mgmt_readdata,
   input wire         reconfig_mgmt_waitrequest,
   output wire [2:0]  gxb_rx_cal_busy_out,
   output wire        rx_reconfig_en,

   input wire[2:0]      gxb_reconfig_write,
   input wire[2:0]      gxb_reconfig_read,
   input wire  [32:0]   gxb_reconfig_address,
   input wire  [95:0]   gxb_reconfig_writedata,
   output wire [95:0]   gxb_reconfig_readdata,
   output wire[2:0]     gxb_reconfig_waitrequest,
   input wire [2:0]     gxb_rx_cal_busy_in,
   
   input wire           edid_ram_access,
   input wire [7:0]     edid_ram_address,
   input wire           edid_ram_write,
   input wire           edid_ram_read,
   output wire [7:0]    edid_ram_readdata,
   input wire [7:0]     edid_ram_writedata,
   output wire          edid_ram_waitrequest
);

wire        rx_hpd;
wire        hdmi_rx_hpd_tmp;
wire        mgmt_clk_reset_sync;
assign hdmi_rx_hpd_tmp   = rx_hpd;


//
// Generate the HPD for source to re-trigger the SCDC write upon power up
//
localparam [31:0] HPD_TIMEOUT_COUNT = 32'd100000000; //hold HPD for 1s
wire    sys_init_int;
wire clr_count;
wire hdmi_rx_5v_detect_sync;

//assign clr_count = ~hdmi_rx_5v_detect_sync | edid_ram_access | mgmt_clk_reset_sync | ~user_pb_0;
assign clr_count = ~hdmi_rx_5v_detect_sync |  ~user_pb_0;

altera_std_synchronizer sync_locked(
    .clk(mgmt_clk), 
    .reset_n(~mgmt_clk_reset_sync), 
    .din(in_5v_power), 
    .dout(hdmi_rx_5v_detect_sync)
);

reg [31:0] hpd_count = 32'd0;
always @ (posedge mgmt_clk or posedge mgmt_clk_reset_sync)
begin
    if(mgmt_clk_reset_sync) begin
        hpd_count <= 32'd0;
    end else begin
        if(clr_count) begin
            hpd_count <= 32'd0;
        end else begin
            if (hpd_count < HPD_TIMEOUT_COUNT) begin 
                hpd_count <= hpd_count + 32'd1;
            end
        end
    end
end
assign rx_hpd = hpd_count > (HPD_TIMEOUT_COUNT - 32'd1);

reg [3:0] sys_init_count = 4'd0;

always @ (posedge mgmt_clk)
begin
    if (sys_init_count < 4'd11) begin
        sys_init_count <= sys_init_count + 4'd1;
    end
end


assign sys_init_int = sys_init_count > 4'd5 && sys_init_count < 4'd10;
assign sys_init = sys_init_int;

//
// RX Display Data Channel (DDC)
// I2C slave and EDID components
//
wire        i2c_clk_reset_sync;
wire [7:0]  edid_rdata;
//wire        edid_w;
wire        edid_r;
wire [31:0] edid_addr;
//wire [31:0] edid_wdata;
wire        edid_sda_oe;
wire        i2c_sda_in;
wire [7:0]  scdc_rdata;
wire        scdc_w;
wire        scdc_r;
wire [31:0] scdc_addr;
wire [31:0] scdc_wdata;
   
reg edid_rdvalid;
always @(posedge i2c_clk)
   edid_rdvalid <= edid_r;
	
i2cslave_to_avlmm_bridge #(
   .I2C_SLAVE_ADDRESS (10'b0001010000),
   .BYTE_ADDRESSING (1),
   .ADDRESS_STEALING (0),
   .READ_ONLY (0)
) u_i2cslave_edid (
   .clk (i2c_clk),
   .rst_n (~i2c_clk_reset_sync),
   .address (edid_addr),
   .read (edid_r),
   .readdata ({24'd0, edid_rdata}),
   .readdatavalid (edid_rdvalid),
   .waitrequest (1'b0),
   .write (),
   .byteenable (),
   .writedata (),
   .i2c_data_in (i2c_sda_in),
   .i2c_clk_in (hdmi_rx_i2c_scl),
   .i2c_data_oe (edid_sda_oe),
   .i2c_clk_oe ()	    
);

wire scdc_sda_oe;
   
reg scdc_rdvalid;
always @(posedge i2c_clk)
   scdc_rdvalid <= scdc_r;
	
i2cslave_to_avlmm_bridge #(
   .I2C_SLAVE_ADDRESS (10'b0001010100),
   .BYTE_ADDRESSING (1),
   .ADDRESS_STEALING (0),
   .READ_ONLY (0)
) u_i2cslave_scdc (
   .clk (i2c_clk),
   .rst_n (~i2c_clk_reset_sync),
   .address (scdc_addr),
   .read (scdc_r),
   .readdata ({24'd0, scdc_rdata}),
   .readdatavalid (scdc_rdvalid),
   .waitrequest	(1'b0),
   .write (scdc_w),
   .byteenable (),
   .writedata (scdc_wdata),
   .i2c_data_in (i2c_sda_in),
   .i2c_clk_in (hdmi_rx_i2c_scl),
   .i2c_data_oe (scdc_sda_oe),
   .i2c_clk_oe ()	    
);

wire SYNTHESIZED_WIRE_3;
assign SYNTHESIZED_WIRE_3 = scdc_sda_oe | edid_sda_oe;
   
output_buf_i2c u_i2c_buf (
   /* I */ .datain (1'b0),
   /* B */ .dataio (hdmi_rx_i2c_sda), 
   /* I */ .oe (SYNTHESIZED_WIRE_3),
   /* O */ .dataout (i2c_sda_in)
);

output_buf_i2c u_rx_hpd_buf (
   /* I */ .datain (1'b0),
   /* B */ .dataio (hdmi_rx_hpd_n), 
   /* I */ .oe (hdmi_rx_hpd_tmp),
   /* O */ .dataout ()
);

wire [7:0] edid_ram_address_tmp;
wire edid_ram_read_tmp;
reg [2:0] edid_ram_read_r;
reg edid_ram_write_r0;

edid_ram u_edid_ram (
   /* I */ .wren ({edid_ram_write & ~edid_ram_write_r0}), //write data change after the first clock cycle of write, write pulse for 2 cycles, so need to limit write to 1 cycle
   /* I */ .rden (edid_ram_read_tmp),
   /* I */ .clock (i2c_clk),
   /* I */ .address (edid_ram_address_tmp),
   /* I */ .data (edid_ram_writedata),
   /* O */ .q (edid_rdata)
);

assign edid_ram_address_tmp = ({8{~edid_ram_access}} & edid_addr[7:0]) | ({8{edid_ram_access}} & edid_ram_address);
assign edid_ram_read_tmp = (~edid_ram_access) | (edid_ram_access & edid_ram_read & ~edid_ram_read_r[0]);
assign edid_ram_readdata = edid_rdata;
assign edid_ram_waitrequest = edid_ram_access & ((edid_ram_read & ~edid_ram_read_r[0]) | {edid_ram_write & ~edid_ram_write_r0});

always @(posedge i2c_clk or posedge i2c_clk_reset_sync)
begin
    if(i2c_clk_reset_sync) begin
        edid_ram_read_r <= 1'b0;
        edid_ram_write_r0 <= 1'b0;
    end else begin
        edid_ram_read_r <= {edid_ram_read_r[1:0], edid_ram_read};
        edid_ram_write_r0 <= edid_ram_write;
    end
end

// Rx Native PHY Transceiver
wire        mgmt_clk_core_reset_sync;
wire        reset_xcvr;
wire        reset_pll;
wire        reset_pll_reconfig;
wire [2:0]  rx_analogreset;
wire [2:0]  rx_analogreset_stat;
wire [2:0]  rx_digitalreset;
wire [2:0]  rx_digitalreset_stat;
wire [2:0]  rx_set_locktoref;
wire [2:0]  rx_freqlocked;
wire [2:0]  rx_is_lockedtoref;
wire [59:0] rx_data;
wire [2:0]  rx_clk; 
wire        cdr_refclk0;
wire [3:0]  color_depth;
wire [3:0]  color_depth_sync;
wire [SYMBOLS_PER_CLOCK*10*3-1:0] rx_parallel_data;
wire [SYMBOLS_PER_CLOCK*10*3-1:0] rx_parallel_data_i;
wire [14:0] rx_std_bitslipboundarysel;
wire        reconfig_pcs_in_progress;

// Need to hold the rx_digital reset until the rx_digitalreset_state asserted
// Full handshaking with the main state machine to remain in RECONFIG_PCS until rx_digitalreset_stat deasserted for all channels
wire [2:0] rx_digitalreset_stat_sync;
genvar ch;
generate 
  for (ch=0;ch<3;ch=ch+1) begin: rx_digitalreset_stat_sync_loop
    altera_std_synchronizer u_rx_digitalreset_stat_sync(
        .clk(mgmt_clk), 
        .reset_n(~mgmt_clk_reset_sync), 
        .din(rx_digitalreset_stat[ch]), 
        .dout(rx_digitalreset_stat_sync[ch])
    );
  end
endgenerate

reg [2:0] reconfig_pcs_rx_reset;
always @(posedge mgmt_clk or posedge mgmt_clk_reset_sync)
begin
  if(mgmt_clk_reset_sync) begin
    reconfig_pcs_rx_reset <= 3'b000;
  end else begin
    // Hold the rx_digitalreset until the assertion of the rx_digitalreset_stat
     if(reconfig_pcs_in_progress) begin
      reconfig_pcs_rx_reset[0] <= 1'b1; 
      reconfig_pcs_rx_reset[1] <= 1'b1; 
      reconfig_pcs_rx_reset[2] <= 1'b1; 
     end else begin
      if(rx_digitalreset_stat_sync[0]) reconfig_pcs_rx_reset[0] <= 1'b0;
      if(rx_digitalreset_stat_sync[1]) reconfig_pcs_rx_reset[1] <= 1'b0;
      if(rx_digitalreset_stat_sync[2]) reconfig_pcs_rx_reset[2] <= 1'b0;
     end
   end
end

gxb_rx u_gxb_rx (
  /* I */ .reconfig_write (gxb_reconfig_write), 
  /* I */ .reconfig_read (gxb_reconfig_read),
  /* I */ .reconfig_address (gxb_reconfig_address),
  /* I */ .reconfig_writedata (gxb_reconfig_writedata),
  /* O */ .reconfig_readdata (gxb_reconfig_readdata),
  /* O */ .reconfig_waitrequest (gxb_reconfig_waitrequest),
  /* I */ .reconfig_clk ({3{mgmt_clk}}),
  /* I */ .reconfig_reset ({3{mgmt_clk_reset_sync}}),
  /* I */ .rx_analogreset (rx_analogreset),
  /* O */ .rx_analogreset_stat (rx_analogreset_stat),
  /* O */ .rx_cal_busy (gxb_rx_cal_busy_out),
  /* I */ .rx_cdr_refclk0 (tmds_clk_in),
  /* O */ .rx_clkout (rx_clk),
  /* I */ .rx_coreclkin (rx_clk),
  /* I */ .rx_digitalreset (rx_digitalreset | reconfig_pcs_rx_reset),
  /* O */ .rx_digitalreset_stat (rx_digitalreset_stat),
  /* O */ .rx_is_lockedtodata (rx_freqlocked), 
  /* O */ .rx_is_lockedtoref (rx_is_lockedtoref),
  /* O */ .rx_parallel_data (rx_parallel_data_i),
  /* I */ .rx_serial_data (rx_serial_data),
  /* I */ .rx_set_locktodata (3'd0), 
  /* I */ .rx_set_locktoref (rx_set_locktoref), 
  /* O */ .unused_rx_parallel_data (),
  /* I */ .rx_polinv ({(POLARITY_INVERSION)? 3'b111 : 3'b000}),
  /* O */ .rx_patterndetect (),
  /* O */ .rx_syncstatus (),
  /* I */ .rx_std_wa_patternalign (3'b000), 
  /* O */ .rx_std_bitslipboundarysel (rx_std_bitslipboundarysel)
);

wire [2:0] rx_datalock;
wire tkn_detected;
wire [SYMBOLS_PER_CLOCK*10*3-1:0] rx_parallel_data_aligned;
wire [2:0] rx_reset_is_lockedtodata;

assign rx_reset_is_lockedtodata = &rx_set_locktoref ? rx_is_lockedtoref : rx_freqlocked;

symbol_aligner #(
  .SYMBOLS_PER_CLOCK (SYMBOLS_PER_CLOCK)
) u_symbol_aligner (
  /* I */ .reset (reset),
  /* I */ .mgmt_clk (mgmt_clk),
  /* I */ .rx_clk (rx_clk),
  /* I */ .rx_parallel_data (rx_parallel_data_i),
  /* I */ .rx_std_bitslipboundarysel (rx_std_bitslipboundarysel),
  /* I */ .rx_ready (rx_reset_is_lockedtodata), //was rx_datalock: rx_ready will go low when rx_digitalreset_stat goes high
  /* I */ .rx_reconfig_pcs_in_progress (reconfig_pcs_in_progress),
  /* O */ .token_detected (tkn_detected),
  /* O */ .rx_parallel_data_aligned (rx_parallel_data_aligned)				 
);
  
// Rx Transceiver Reset Controller


gxb_rx_reset u_gxb_rx_reset (
  /* I */ .clock (mgmt_clk),
  /* I */ .reset (reset_xcvr),
  /* O */ .rx_analogreset (rx_analogreset),
  /* I */ .rx_analogreset_stat (rx_analogreset_stat),
  /* I */ .rx_cal_busy (gxb_rx_cal_busy_in), 
  /* O */ .rx_digitalreset (rx_digitalreset),
  /* O */ .rx_digitalreset_stat (rx_digitalreset_stat),
  /* I */ .rx_is_lockedtodata (rx_reset_is_lockedtodata), 
  /* O */ .rx_ready (rx_datalock)			     
);

assign gxb_rx_ready = &rx_datalock;
   
// HDMI Rx core
wire       ls_clk;
wire       vid_clk;
wire [2:0] rx_core_ready;
wire       reset_core;

assign rx_core_ready = rx_datalock & {3{~reset_core}};

reg [SYMBOLS_PER_CLOCK*10*3-1:0] rx_parallel_data_r0;
genvar l;
generate 
begin: pipepine
  for(l=0;l<3;l=l+1)
  begin: pipeline_loop
    always @(posedge rx_clk[l])
    begin
      rx_parallel_data_r0[(SYMBOLS_PER_CLOCK*10*l)+:(SYMBOLS_PER_CLOCK*10)] <= rx_parallel_data_i[(SYMBOLS_PER_CLOCK*10*l)+:(SYMBOLS_PER_CLOCK*10)];
    end
  end
end
endgenerate


wire [SYMBOLS_PER_CLOCK*10-1:0] r,g,b;
wire [2:0] rx_clk_remap;
wire clk_r, clk_g, clk_b;

generate
if (BITEC_DAUGHTER_CARD_REV==11) begin: BITEC_RX_MAP
   assign {b,r,g} = rx_parallel_data_r0;
   assign rx_parallel_data = {r,g,b};
   assign {clk_b, clk_r, clk_g} = rx_clk;
   assign rx_clk_remap = {clk_r, clk_g, clk_b};
end else if (BITEC_DAUGHTER_CARD_REV==4 || BITEC_DAUGHTER_CARD_REV==6) begin 
   assign {r,b,g} = rx_parallel_data_r0;
   assign rx_parallel_data = {r,g,b};
   assign {clk_r, clk_b, clk_g} = rx_clk;
   assign rx_clk_remap = {clk_r, clk_g, clk_b};
end else begin
   assign {r,g,b} = rx_parallel_data_r0;
   assign rx_parallel_data = {r,g,b};
   assign {clk_r, clk_g, clk_b} = rx_clk;
   assign rx_clk_remap = {clk_r, clk_g, clk_b};
end
endgenerate

wire [2:0] rx_core_ready_sync;
clock_crosser #(.W(1)) cc_ready_sync0 (.in(rx_core_ready[0]),.out(rx_core_ready_sync[0]),.in_clk(mgmt_clk),.out_clk(ls_clk),.in_reset(1'b0),.out_reset(1'b0));
clock_crosser #(.W(1)) cc_ready_sync1 (.in(rx_core_ready[1]),.out(rx_core_ready_sync[1]),.in_clk(mgmt_clk),.out_clk(ls_clk),.in_reset(1'b0),.out_reset(1'b0));
clock_crosser #(.W(1)) cc_ready_sync2 (.in(rx_core_ready[2]),.out(rx_core_ready_sync[2]),.in_clk(mgmt_clk),.out_clk(ls_clk),.in_reset(1'b0),.out_reset(1'b0));

//wire       os;
//wire [7:0] scdc_rdata;
//wire       scdc_w;
//wire       scdc_r;
//wire [7:0] scdc_addr;
//wire [7:0] scdc_wdata;
//wire       i2c_clk;
mr_hdmi_rx_core_top u_hdmi_rx_core_top (
   /* I */ .reset ({mgmt_clk_core_reset_sync}), // was mgmt_clk_reset_sync | ~(&rx_datalock && iopll_locked)
   /* I */ .reset_vid ({mgmt_clk_core_reset_sync}),
   /* I */ .rx_clk (rx_clk_remap),
   /* I */ .ls_clk (ls_clk),  	    
   /* I */ .vid_clk (vid_clk),  
   /* I */ .os (os),
   /* I */ .rx_parallel_data (rx_parallel_data),
   /* I */ .rx_datalock (rx_core_ready_sync), 
   /* O */ .aux_data (aux_data),
   /* O */ .aux_valid (aux_valid),
   /* O */ .aux_sop (aux_sop),
   /* O */ .aux_eop (aux_eop), 
   /* O */ .aux_error (aux_error),
   /* O */ .gcp (gcp),
   /* O */ .info_avi (info_avi),
   /* O */ .info_vsi (info_vsi),
   /* O */ .aux_pkt_addr (aux_pkt_addr),
   /* O */ .aux_pkt_data (aux_pkt_data),
   /* O */ .aux_pkt_wr (aux_pkt_wr),
   /* O */ .audio_CTS (audio_CTS), 
   /* O */ .audio_N (audio_N), 
   /* O */ .audio_data (audio_data), 
   /* O */ .audio_de (audio_de),
   /* O */ .audio_metadata (audio_metadata),
   /* O */ .audio_format (audio_format),   
   /* O */ .audio_info_ai (audio_info_ai),
   /* O */ .locked (locked),
   /* O */ .vid_data (vid_data),
   /* O */ .vid_de (vid_de),
   /* O */ .vid_hsync (vid_hsync),
   /* O */ .vid_vsync (vid_vsync),
   /* O */ .vid_lock (vid_lock),
   /* I */ .in_5v_power (in_5v_power), 
   /* I */ .in_hpd (hdmi_rx_hpd_tmp), 
   /* O */ .mode (mode),
   /* O */ .ctrl (ctrl),					
   /* I */ .scdc_i2c_clk (i2c_clk),
   /* I */ .scdc_i2c_addr (scdc_addr[7:0]),
   /* O */ .scdc_i2c_rdata (scdc_rdata),
   /* I */ .scdc_i2c_wdata (scdc_wdata[7:0]),
   /* I */ .scdc_i2c_r (scdc_r),
   /* I */ .scdc_i2c_w (scdc_w),
   /* O */ .TMDS_Bit_clock_Ratio (TMDS_Bit_clock_Ratio));
defparam u_hdmi_rx_core_top.SUPPORT_AUDIO = SUPPORT_AUDIO;
defparam u_hdmi_rx_core_top.SUPPORT_AUXILIARY = SUPPORT_AUXILIARY;
defparam u_hdmi_rx_core_top.SUPPORT_DEEP_COLOR = SUPPORT_DEEP_COLOR;
defparam u_hdmi_rx_core_top.SYMBOLS_PER_CLOCK = SYMBOLS_PER_CLOCK;
   
// The IO PLL is defaulted to the following settings to ease timing constraint/analysis
// The default setting is not optimum for real time run, reconfig to the right value per incoming
// stream rate will be performed (taken care by state machine in multi-rate operation) upon power-up.
// refclk = 150MHz (for rate detect 24-bit counter)
// outclk_0 = 300MHz (ls clk)
// outclk_1 = 300MHz (vid clk)
//wire pll_locked;
wire [29:0] reconfig_to_pll;
wire [10:0] reconfig_from_pll;
pll_hdmi u_iopll_rx (
   /* I */ .refclk (tmds_clk_in), 
   /* I */ .rst (reset_pll),
   /* O */ .outclk_0 (ls_clk), 
   /* O */ .outclk_1 (vid_clk),
   /* O */ .locked (iopll_locked),	
   /* I */ .reconfig_to_pll (reconfig_to_pll), 
   /* O */ .reconfig_from_pll (reconfig_from_pll)	
);

assign vid_clk_out = vid_clk;
assign ls_clk_out = ls_clk;
  
// PLL reconfig controller
wire        pll_reconfig_waitrequest;
wire [9:0]  pll_reconfig_address;
wire        pll_reconfig_write;
wire [7:0] pll_reconfig_writedata;
wire        pll_reconfig_read;
wire [7:0] pll_reconfig_readdata;

pll_hdmi_reconfig u_iopll_reconfig (
   /* I */ .mgmt_clk (mgmt_clk),
   /* I */ .mgmt_reset (reset_pll_reconfig), 
   /* O */ .mgmt_waitrequest (pll_reconfig_waitrequest),
   /* I */ .mgmt_read (pll_reconfig_read),
   /* I */ .mgmt_write (pll_reconfig_write), 
   /* O */ .mgmt_readdata (pll_reconfig_readdata),
   /* I */ .mgmt_address (pll_reconfig_address),
   /* I */ .mgmt_writedata (pll_reconfig_writedata),
   /* O */ .reconfig_to_pll (reconfig_to_pll),
   /* I */ .reconfig_from_pll (reconfig_from_pll)
);   
 
assign color_depth = gcp[3:0];
clock_crosser #(.W(4)) cc_cd (.in(color_depth), .out(color_depth_sync), .in_clk(ls_clk),.out_clk(mgmt_clk),.in_reset(1'b0),.out_reset(1'b0));
assign colordepth_mgmt_sync = color_depth_sync[1:0]; // only bit 1 and 0 used by CPU
wire measure_valid_reconfig_mgmt;

wire i2c_trans_detected_ack;
wire i2c_trans_detected_ack_sync;
reg i2c_trans_detected;
always @(posedge i2c_clk or posedge i2c_clk_reset_sync)
begin
  if(i2c_clk_reset_sync) begin
    i2c_trans_detected <= 1'b0;
  end else begin
    if(scdc_w & (scdc_addr == 8'h20) & ((scdc_wdata[1:0] == 2'b11) | (scdc_wdata[1:0] == 2'b00)))
      i2c_trans_detected <= 1'b1;
    else if(i2c_trans_detected_ack_sync)
      i2c_trans_detected <= 1'b0;
  end
end

wire i2c_trans_detected_sync;
clock_crosser #(.W(1)) cc_i2c_trans_detected_sync (.in(i2c_trans_detected), .out(i2c_trans_detected_sync), .in_clk(i2c_clk),.out_clk(mgmt_clk),.in_reset(1'b0),.out_reset(1'b0));
clock_crosser #(.W(1)) cc_i2c_trans_detected_ack_sync (.in(i2c_trans_detected_ack), .out(i2c_trans_detected_ack_sync), .in_clk(mgmt_clk),.out_clk(i2c_clk),.in_reset(1'b0),.out_reset(1'b0));

mr_reconfig_mgmt u_mr_reconfig_mgmt (
  /* I */ .clock (mgmt_clk),
  /* I */ .reset (mgmt_clk_reset_sync),
  /* I */ .refclock (tmds_clk_in), 
  /* I */ .rx_is_20 (TMDS_Bit_clock_Ratio),  
  /* I */ .rx_hdmi_locked (locked),
  /* I */ .rx_color_depth (color_depth_sync),  
  /* I */ .rx_ready (&rx_datalock),
  /* I */ .pll_locked (iopll_locked),
  /* O */ .reset_core (reset_core), 
  /* O */ .reset_xcvr (reset_xcvr),
  /* O */ .reset_pll (reset_pll),
  /* O */ .reset_pll_reconfig (reset_pll_reconfig),
  /* O */ .rx_set_locktoref (rx_set_locktoref),
  /* O */ .measure (measure),
  /* O */ .measure_valid (measure_valid_reconfig_mgmt),
  /* I */ .rx_cal_busy (gxb_rx_cal_busy_in),
  /* I */ .rx_reconfig_waitrequest (reconfig_mgmt_waitrequest),
  /* I */ .rx_reconfig_readdata (reconfig_mgmt_readdata),
  /* O */ .rx_reconfig_write (reconfig_mgmt_write),
  /* O */ .rx_reconfig_read (reconfig_mgmt_read),
  /* O */ .rx_reconfig_address (reconfig_mgmt_address),
  /* O */ .rx_reconfig_writedata (reconfig_mgmt_writedata),
  /* I */ .pll_reconfig_waitrequest (pll_reconfig_waitrequest),
  /* O */ .pll_reconfig_write (pll_reconfig_write),
  /* O */ .pll_reconfig_address (pll_reconfig_address),
  /* O */ .pll_reconfig_writedata (pll_reconfig_writedata),
  /* 0 */ .pll_reconfig_read (pll_reconfig_read),
  /* I */ .pll_reconfig_readdata (pll_reconfig_readdata),
  /* O */ .rx_reconfig_en (rx_reconfig_en),
  /* I */ .i2c_trans_detected (i2c_trans_detected_sync),
  /* O */ .i2c_trans_detected_ack (i2c_trans_detected_ack),
  /* I */ .tkn_detected (tkn_detected),
  /* O */ .reconfig_pcs_in_progress (reconfig_pcs_in_progress)
);	
defparam u_mr_reconfig_mgmt.FAST_SIMULATION = 0;
   
assign os = &rx_set_locktoref;

wire locked_sync;
clock_crosser #(.W(1)) cc_hdmi_rx_locked (.in(locked), .out(locked_sync), .in_clk(ls_clk),.out_clk(mgmt_clk),.in_reset(1'b0),.out_reset(1'b0));
assign measure_valid = measure_valid_reconfig_mgmt & (locked_sync);

//
// Reset synchronizers
//
altera_reset_controller #(
   .NUM_RESET_INPUTS          (3),
   .SYNC_DEPTH                (3),
   .RESET_REQ_WAIT_TIME       (1),
   .MIN_RST_ASSERTION_TIME    (3),
   .RESET_REQ_EARLY_DSRT_TIME (1)
) u_mgmt_clk_reset_sync_sync (
   /* I */ .reset_in0 (reset),
   /* I */ .reset_in1 (sys_init_int),
   /* I */ .reset_in2 (edid_ram_access),
   /* I */ .clk (mgmt_clk),
   /* O */ .reset_out (mgmt_clk_reset_sync)
);

altera_reset_controller #(
   .NUM_RESET_INPUTS          (2),
   .SYNC_DEPTH                (3),
   .RESET_REQ_WAIT_TIME       (1),
   .MIN_RST_ASSERTION_TIME    (3),
   .RESET_REQ_EARLY_DSRT_TIME (1)
) u_mgmt_clk_core_reset_sync (
   /* I */ .reset_in0 (reset),
   /* I */ .reset_in1 (sys_init_int),
   /* I */ .clk (mgmt_clk),
   /* O */ .reset_out (mgmt_clk_core_reset_sync)
);

altera_reset_controller #(
   .NUM_RESET_INPUTS          (1),
   .SYNC_DEPTH                (3),
   .RESET_REQ_WAIT_TIME       (1),
   .MIN_RST_ASSERTION_TIME    (3),
   .RESET_REQ_EARLY_DSRT_TIME (1)
) u_i2c_clk_reset_sync (
   /* I */ .reset_in0 (reset),
   /* I */ .clk (i2c_clk),
   /* O */ .reset_out (i2c_clk_reset_sync)
);


//////////////////////////////////////////////////////
// Measure circuitry for clock

`ifdef DEBUG_EN

mr_rate_detect #(		 
    .CYC_MEASURE_CLK_IN_10_MSEC (1000000) 
) u_rate_detect_tmds_clk (
    /* I */ .refclock         (tmds_clk_in), 
    /* I */ .measure_clk      (mgmt_clk),
    /* I */ .reset            (reset),
    /* I */ .enable           (1'b1),    
    /* O */ .refclock_measure (),
    /* O */ .valid            ()
);


mr_rate_detect #(		 
    .CYC_MEASURE_CLK_IN_10_MSEC (1000000) 
) u_rate_detect_rx_clkout (
    /* I */ .refclock         (rx_clk[0]), 
    /* I */ .measure_clk      (mgmt_clk),
    /* I */ .reset            (reset),
    /* I */ .enable           (1'b1),    
    /* O */ .refclock_measure (),
    /* O */ .valid            ()
);

mr_rate_detect #(		 
    .CYC_MEASURE_CLK_IN_10_MSEC (1000000) 
) u_rate_detect_rx_ls_clk (
    /* I */ .refclock         (ls_clk), 
    /* I */ .measure_clk      (mgmt_clk),
    /* I */ .reset            (reset),
    /* I */ .enable           (1'b1),    
    /* O */ .refclock_measure (),
    /* O */ .valid            ()
);

mr_rate_detect #(		 
    .CYC_MEASURE_CLK_IN_10_MSEC (1000000) 
) u_rate_detect_rx_vid_clk (
    /* I */ .refclock         (vid_clk), 
    /* I */ .measure_clk      (mgmt_clk),
    /* I */ .reset            (reset),
    /* I */ .enable           (1'b1),    
    /* O */ .refclock_measure (),
    /* O */ .valid            ()
);

wire vid_lock_sync;
clock_crosser #(.W(1)) cc_vid_lock_sync (.in(vid_lock), .out(vid_lock_sync), .in_clk(ls_clk),.out_clk(mgmt_clk),.in_reset(1'b0),.out_reset(1'b0));


reg [31:0] debug_cntr /* synthesis noprune */;
reg [31:0] debug_ms_cntr /* synthesis noprune */;
reg debug_ms_incr /* synthesis preserve */;
always @(posedge mgmt_clk) begin
  debug_ms_incr   <= (debug_cntr == 32'h186A0);

  if(vid_lock_sync && locked_sync) begin
    debug_cntr <= 'b0;
    debug_ms_incr <= 'b0;
  end else begin
    if (debug_ms_incr) begin
        debug_cntr <= 'b0;
    end else begin
        debug_cntr  <= debug_cntr + 1;
    end

    if (debug_ms_incr) begin
        debug_ms_cntr <= debug_ms_cntr + 1'b1;
    end
  end
end

`endif

endmodule

