// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



module mr_hdmi_rx_core_top #(
    parameter SUPPORT_DEEP_COLOR  = 1,
    parameter SUPPORT_AUXILIARY   = 1,
    parameter SYMBOLS_PER_CLOCK   = 2,
    parameter SUPPORT_AUDIO       = 1
) (
    input  wire                                    reset,			 
    input  wire                                    reset_vid,
    input  wire [2:0]                              rx_clk,
    input  wire                                    ls_clk,
    input  wire                                    vid_clk,			 
    input  wire                                    os,
    input  wire [SYMBOLS_PER_CLOCK*10*3-1:0]       rx_parallel_data,
    input  wire [2:0]                              rx_datalock,
    input  wire                                    scdc_i2c_clk,
    input  wire [7:0]                              scdc_i2c_addr,
    output wire [7:0]                              scdc_i2c_rdata,
    input  wire [7:0]                              scdc_i2c_wdata,
    input  wire                                    scdc_i2c_r,
    input  wire                                    scdc_i2c_w,
    output wire                                    TMDS_Bit_clock_Ratio,
    output reg                                    audio_de,
    output reg [255:0]                            audio_data,
    output reg [47:0]                             audio_info_ai,   
    output reg [19:0]                             audio_N,
    output reg [19:0]                             audio_CTS,
    output reg [164:0]                            audio_metadata,
    output reg [4:0]                              audio_format,
    output reg [71:0]                             aux_pkt_data,
    output reg [6:0]                              aux_pkt_addr,
    output reg                                    aux_pkt_wr,
    output reg [5:0]                              gcp,
    output reg [111:0]                            info_avi,
    output reg [60 :0]                            info_vsi,
    output reg [71:0]                             aux_data,
    output reg                                    aux_sop,
    output reg                                    aux_eop,
    output reg                                    aux_valid,
    output reg                                    aux_error,
    output reg                                    locked,
    output reg [SYMBOLS_PER_CLOCK*48-1:0]         vid_data,
    output reg [SYMBOLS_PER_CLOCK-1:0]            vid_vsync,
    output reg [SYMBOLS_PER_CLOCK-1:0]            vid_hsync,
    output reg [SYMBOLS_PER_CLOCK-1:0]            vid_de,
    output reg                                    vid_lock,
    input  wire                                    in_5v_power,
    input  wire                                    in_hpd,
    output wire                                    mode,
    output wire [SYMBOLS_PER_CLOCK*6-1:0]          ctrl
);

genvar i;
wire [2:0] rx_clk_rst_sync;
generate
    for (i=0; i<3; i=i+1) begin: RX_CLK_RST_SYNCHRONIZER
        altera_reset_controller #(
            .NUM_RESET_INPUTS (2),
            .SYNC_DEPTH (3)				
        ) u_rx_clk_rst_sync (
            /* I */ .reset_in0 (reset),
            /* I */ .reset_in1 ({~&rx_datalock}),
            /* I */ .clk (rx_clk[i]),
            /* O */ .reset_out (rx_clk_rst_sync[i])
        );
    end 
endgenerate
   
wire [SYMBOLS_PER_CLOCK*10*3-1:0] rx_oversample_out;
wire [2:0]                        rx_oversample_out_valid; 
wire [2:0]                        rx_fifo_wrreq;
reg [2:0]                        rx_fifo_wrreq_r0;
wire [SYMBOLS_PER_CLOCK*10*3-1:0] rx_fifo_in;
reg [SYMBOLS_PER_CLOCK*10*3-1:0] rx_fifo_in_r0;
wire [SYMBOLS_PER_CLOCK*10*3-1:0] rx_fifo_out;                
       
generate
    for (i=0; i<3; i=i+1) begin: RX_OVERSAMPLE   
        mr_rx_oversample #(
            .DIN_WIDTH (SYMBOLS_PER_CLOCK*10),			    
            .DOUT_WIDTH (SYMBOLS_PER_CLOCK*10),
            .SAMPLE_INTERVAL (5),
            .SAMPLE_MASK (5'b00100)		   
        ) u_rx_os (
            /* I */ .clk (rx_clk[i]),
            /* I */ .rst (rx_clk_rst_sync[i]),
            /* I */ .din (rx_parallel_data[i*SYMBOLS_PER_CLOCK*10+SYMBOLS_PER_CLOCK*10-1:i*SYMBOLS_PER_CLOCK*10]),
            /* O */ .dout (rx_oversample_out[i*SYMBOLS_PER_CLOCK*10+SYMBOLS_PER_CLOCK*10-1:i*SYMBOLS_PER_CLOCK*10]),
            /* O */ .dout_valid (rx_oversample_out_valid[i])       
        );
    end
endgenerate

wire ls_clk_rst_sync;
altera_reset_controller #(
    .NUM_RESET_INPUTS (2),
    .SYNC_DEPTH (3)			  
) u_ls_clk_rst_sync (
    /* I */ .reset_in0 (reset),
    /* I */ .reset_in1 ({~&rx_datalock}),
    /* I */ .clk (ls_clk),
    /* O */ .reset_out (ls_clk_rst_sync)
);
   
assign rx_fifo_in = os ? rx_oversample_out : rx_parallel_data;
assign rx_fifo_wrreq = os ? rx_oversample_out_valid : 3'b111;
   
generate
    for (i=0; i<3; i=i+1) begin: RX_DCFIFO   
    
        always @(posedge rx_clk[i] or posedge rx_clk_rst_sync[i])
        begin
          if(rx_clk_rst_sync[i]) begin
            rx_fifo_wrreq_r0[i] <= 1'b0;
            rx_fifo_in_r0[i*SYMBOLS_PER_CLOCK*10+SYMBOLS_PER_CLOCK*10-1:i*SYMBOLS_PER_CLOCK*10] <= {(SYMBOLS_PER_CLOCK*10){1'b0}};
          end else begin
            rx_fifo_wrreq_r0[i] <= rx_fifo_wrreq[i];
            rx_fifo_in_r0[i*SYMBOLS_PER_CLOCK*10+SYMBOLS_PER_CLOCK*10-1:i*SYMBOLS_PER_CLOCK*10] <= rx_fifo_in[i*SYMBOLS_PER_CLOCK*10+SYMBOLS_PER_CLOCK*10-1:i*SYMBOLS_PER_CLOCK*10];
          end
        end
        
        mr_clock_sync #(
            .DEVICE_FAMILY ("Arria 10"),
            .SYMBOLS_PER_CLOCK (SYMBOLS_PER_CLOCK)			 
        ) u_rx_clock_sync (
            /* I */ .wrclk (rx_clk[i]),
            /* I */ .rdclk (ls_clk),
            /* I */ .aclr (ls_clk_rst_sync), 
            /* I */ .wrreq (rx_fifo_wrreq_r0[i]),
            /* I */ .data (rx_fifo_in_r0[i*SYMBOLS_PER_CLOCK*10+SYMBOLS_PER_CLOCK*10-1:i*SYMBOLS_PER_CLOCK*10]),
            /* I */ .rdreq (1'b1),
            /* O */ .q (rx_fifo_out[i*SYMBOLS_PER_CLOCK*10+SYMBOLS_PER_CLOCK*10-1:i*SYMBOLS_PER_CLOCK*10])		      
        );
    end
endgenerate

wire in_5v_power_sync;
wire in_hpd_sync;
altera_std_synchronizer #(.depth(3)) u_in_5v_power_sync (.clk(scdc_i2c_clk),.reset_n(1'b1),.din(in_5v_power),.dout(in_5v_power_sync));
altera_std_synchronizer #(.depth(3)) u_in_hpd_sync (.clk(scdc_i2c_clk),.reset_n(1'b1),.din(in_hpd),.dout(in_hpd_sync));

reg [SYMBOLS_PER_CLOCK*10*3-1:0] rx_fifo_out_r0;



always @(posedge ls_clk)
begin
	rx_fifo_out_r0 <= rx_fifo_out;
end

wire                                    audio_de_w;
wire [255:0]                            audio_data_w;
wire [47:0]                             audio_info_ai_w;   
wire [19:0]                             audio_N_w;
wire [19:0]                             audio_CTS_w;
wire [164:0]                            audio_metadata_w;
wire [4:0]                              audio_format_w;
wire [71:0]                             aux_pkt_data_w;
wire [6:0]                              aux_pkt_addr_w;
wire                                    aux_pkt_wr_w;
wire [5:0]                              gcp_w;
wire [111:0]                            info_avi_w;
wire [60 :0]                            info_vsi_w;
wire [71:0]                             aux_data_w;
wire                                    aux_sop_w;
wire                                    aux_eop_w;
wire                                    aux_valid_w;
wire                                    aux_error_w;
wire                                    locked_w;
wire [SYMBOLS_PER_CLOCK*48-1:0]         vid_data_w;
wire [SYMBOLS_PER_CLOCK-1:0]            vid_vsync_w;
wire [SYMBOLS_PER_CLOCK-1:0]            vid_hsync_w;
wire [SYMBOLS_PER_CLOCK-1:0]            vid_de_w;
wire                                    vid_lock_w;

//bitec_hdmi_rx u_bitec_hdmi_rx (
hdmi_rx u_hdmi_rx (   
    /* I */ .reset (reset), // was ls_clk_rst_sync
    /* I */ .reset_vid (reset_vid),
    /* I */ .vid_clk (vid_clk),
    /* I */ .ls_clk (ls_clk),						
    /* I */ .in_b (rx_fifo_out_r0[SYMBOLS_PER_CLOCK*10*1-1:0]), 
    /* I */ .in_g (rx_fifo_out_r0[SYMBOLS_PER_CLOCK*10*2-1:SYMBOLS_PER_CLOCK*10*1]),
    /* I */ .in_r (rx_fifo_out_r0[SYMBOLS_PER_CLOCK*10*3-1:SYMBOLS_PER_CLOCK*10*2]), 	 
    /* I */ .in_lock (&rx_datalock[2:0]),
    /* O */ .vid_data (vid_data_w),
    /* O */ .vid_de (vid_de_w),
    /* O */ .vid_hsync (vid_hsync_w),
    /* O */ .vid_vsync (vid_vsync_w),
    /* O */ .vid_lock (vid_lock_w),
    /* O */ .audio_N (audio_N_w),
    /* O */ .audio_CTS (audio_CTS_w),
    /* O */ .audio_data (audio_data_w),		
    /* O */ .audio_de (audio_de_w),
    /* O */ .audio_metadata (audio_metadata_w),
    /* O */ .audio_format (audio_format_w),
    /* O */ .audio_info_ai (audio_info_ai_w),
    /* O */ .aux_pkt_data (aux_pkt_data_w),
    /* O */ .aux_pkt_addr (aux_pkt_addr_w),
    /* O */ .aux_pkt_wr (aux_pkt_wr_w),
    /* O */ .aux_data (aux_data_w),
    /* O */ .aux_sop (aux_sop_w),
    /* O */ .aux_eop (aux_eop_w),
    /* O */ .aux_valid (aux_valid_w),
    /* O */ .aux_error (aux_error_w),
    /* O */ .gcp (gcp_w),	
    /* O */ .info_avi (info_avi_w),
    /* O */ .info_vsi (info_vsi_w),
    /* O */ .locked (locked_w),
    /* I */ .in_5v_power (in_5v_power_sync), 
    /* I */ .in_hpd (in_hpd_sync), 
    /* O */ .mode (mode),
    /* O */ .ctrl (ctrl),		      
    /* I */ .scdc_i2c_clk (scdc_i2c_clk),
    /* I */ .scdc_i2c_addr (scdc_i2c_addr),
    /* O */ .scdc_i2c_rdata (scdc_i2c_rdata),
    /* I */ .scdc_i2c_wdata (scdc_i2c_wdata),
    /* I */ .scdc_i2c_r (scdc_i2c_r),
    /* I */ .scdc_i2c_w (scdc_i2c_w),
    /* O */ .TMDS_Bit_clock_Ratio (TMDS_Bit_clock_Ratio)
);
//defparam u_bitec_hdmi_rx.SUPPORT_8CHAN_AUDIO = SUPPORT_8CHAN_AUDIO;
//defparam u_bitec_hdmi_rx.SUPPORT_AUDIO = SUPPORT_AUDIO;
//defparam u_bitec_hdmi_rx.SUPPORT_AUXILIARY = SUPPORT_AUXILIARY;
//defparam u_bitec_hdmi_rx.SUPPORT_DEEP_COLOR = SUPPORT_DEEP_COLOR;
//defparam u_bitec_hdmi_rx.SYMBOLS_PER_CLOCK = SYMBOLS_PER_CLOCK;

//register output to improve timing
always @(posedge vid_clk)
begin
  vid_data <= vid_data_w;
  vid_vsync <= vid_vsync_w;
  vid_hsync <= vid_hsync_w;
  vid_de <= vid_de_w;
  vid_lock <= vid_lock_w;
end
   
always @(posedge ls_clk)
begin
  audio_de <= audio_de_w;
  audio_data <= audio_data_w;
  audio_info_ai <= audio_info_ai_w;
  audio_N <= audio_N_w;
  audio_CTS <= audio_CTS_w;
  audio_metadata <= audio_metadata_w;
  audio_format <= audio_format_w;
  aux_pkt_data <= aux_pkt_data_w;
  aux_pkt_addr <= aux_pkt_addr_w;
  aux_pkt_wr <= aux_pkt_wr_w;
  gcp <= gcp_w;
  info_avi <= info_avi_w;
  info_vsi <= info_vsi_w;
  aux_data <= aux_data_w;
  aux_sop <= aux_sop_w;
  aux_eop <= aux_eop_w;
  aux_valid <= aux_valid_w;
  aux_error <= aux_error_w;
  locked <= locked_w;
end
   
endmodule			

