// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//
//  GXB RX comparison circuitry that sequentially check the measured value against the predetermined threshold
//  and output the expected start offset pointer 
// 
module mr_compare_rx #(
    parameter       FAST_SIMULATION  = 0,
    parameter [3:0] MIF_OFFSET       = 4,  // ROM_DEPTH_FOR_EACH_MIF_RANGE			
    parameter       POINTER_WIDTH    = 10,  // alt_clogb2(TOTAL_ROM_DEPTH_FOR_VALUEMASK)			  
    parameter       LOOPS_MAX        = 120,
    parameter       LOOPS_WIDTH      = 8   // alt_clogb2(LOOPS_MAX)
) (
    input  wire                     clock,
    input  wire                     reset,
    input  wire                     clr_valid,
    input  wire [23:0]              measure,
    input  wire                     measure_valid,
    output wire [POINTER_WIDTH-1:0] offset_pointer,
    output wire                     offset_pointer_valid,
    output wire [LOOPS_WIDTH-1:0]   range,   
    output wire                     oversampled
);
   
// **** PERL SCIPT TO GENERATE BEGIN **** //
// 1.4 threshold (250Mbps to 3400Mbps & equivalent TMDS clock 25MHz to 340MHz)
localparam [23:0]	 
    THRESHOLD_0    = FAST_SIMULATION ? 24'd 300 : 24'd 310000,
    THRESHOLD_1    = FAST_SIMULATION ? 24'd 520 : 24'd 520000,
    THRESHOLD_2    = FAST_SIMULATION ? 24'd 550 : 24'd 550000,
    THRESHOLD_3    = FAST_SIMULATION ? 24'd 580 : 24'd 580000,
    THRESHOLD_4    = FAST_SIMULATION ? 24'd 600 : 24'd 600000,
    THRESHOLD_5    = FAST_SIMULATION ? 24'd 610 : 24'd 610000,
    THRESHOLD_6    = FAST_SIMULATION ? 24'd 650 : 24'd 650000,
    THRESHOLD_7    = FAST_SIMULATION ? 24'd 680 : 24'd 680000,
    THRESHOLD_8    = FAST_SIMULATION ? 24'd 700 : 24'd 700000,
    THRESHOLD_9    = FAST_SIMULATION ? 24'd 710 : 24'd 710000,
    THRESHOLD_10   = FAST_SIMULATION ? 24'd 750 : 24'd 750000,
    THRESHOLD_11   = FAST_SIMULATION ? 24'd 780 : 24'd 780000,
    THRESHOLD_12   = FAST_SIMULATION ? 24'd 800 : 24'd 800000,
    THRESHOLD_13   = FAST_SIMULATION ? 24'd 810 : 24'd 810000,
    THRESHOLD_14   = FAST_SIMULATION ? 24'd 850 : 24'd 850000,
    THRESHOLD_15   = FAST_SIMULATION ? 24'd 880 : 24'd 880000,
    THRESHOLD_16   = FAST_SIMULATION ? 24'd 900 : 24'd 900000,
    THRESHOLD_17   = FAST_SIMULATION ? 24'd 910 : 24'd 910000,
    THRESHOLD_18   = FAST_SIMULATION ? 24'd 950 : 24'd 950000,
    THRESHOLD_19   = FAST_SIMULATION ? 24'd 980 : 24'd 980000,
    THRESHOLD_20   = FAST_SIMULATION ? 24'd1000 : 24'd1000000,
    THRESHOLD_21   = FAST_SIMULATION ? 24'd1010 : 24'd1010000,
    THRESHOLD_22   = FAST_SIMULATION ? 24'd1050 : 24'd1050000,
    THRESHOLD_23   = FAST_SIMULATION ? 24'd1080 : 24'd1080000,
    THRESHOLD_24   = FAST_SIMULATION ? 24'd1100 : 24'd1100000,
    THRESHOLD_25   = FAST_SIMULATION ? 24'd1130 : 24'd1130000,
    THRESHOLD_26   = FAST_SIMULATION ? 24'd1140 : 24'd1140000,
    THRESHOLD_27   = FAST_SIMULATION ? 24'd1180 : 24'd1180000,
    THRESHOLD_28   = FAST_SIMULATION ? 24'd1200 : 24'd1200000,
    THRESHOLD_29   = FAST_SIMULATION ? 24'd1230 : 24'd1230000,
    THRESHOLD_30   = FAST_SIMULATION ? 24'd1250 : 24'd1250000,
    THRESHOLD_31   = FAST_SIMULATION ? 24'd1280 : 24'd1280000,
    THRESHOLD_32   = FAST_SIMULATION ? 24'd1290 : 24'd1290000,
    THRESHOLD_33   = FAST_SIMULATION ? 24'd1300 : 24'd1300000,
    THRESHOLD_34   = FAST_SIMULATION ? 24'd1330 : 24'd1330000,
    THRESHOLD_35   = FAST_SIMULATION ? 24'd1350 : 24'd1350000,
    THRESHOLD_36   = FAST_SIMULATION ? 24'd1380 : 24'd1380000,
    THRESHOLD_37   = FAST_SIMULATION ? 24'd1400 : 24'd1400000,
    THRESHOLD_38   = FAST_SIMULATION ? 24'd1430 : 24'd1430000,
    THRESHOLD_39   = FAST_SIMULATION ? 24'd1450 : 24'd1450000,
    THRESHOLD_40   = FAST_SIMULATION ? 24'd1480 : 24'd1480000,
    THRESHOLD_41   = FAST_SIMULATION ? 24'd1500 : 24'd1500000,
    THRESHOLD_42   = FAST_SIMULATION ? 24'd1520 : 24'd1520000,
    THRESHOLD_43   = FAST_SIMULATION ? 24'd1550 : 24'd1550000,
    THRESHOLD_44   = FAST_SIMULATION ? 24'd1580 : 24'd1580000,
    THRESHOLD_45   = FAST_SIMULATION ? 24'd1600 : 24'd1600000,
    THRESHOLD_46   = FAST_SIMULATION ? 24'd1630 : 24'd1630000,
    THRESHOLD_47   = FAST_SIMULATION ? 24'd1650 : 24'd1650000,
    THRESHOLD_48   = FAST_SIMULATION ? 24'd1680 : 24'd1680000,
    THRESHOLD_49   = FAST_SIMULATION ? 24'd1700 : 24'd1700000,
    THRESHOLD_50   = FAST_SIMULATION ? 24'd1730 : 24'd1730000,
    THRESHOLD_51   = FAST_SIMULATION ? 24'd1750 : 24'd1750000,
    THRESHOLD_52   = FAST_SIMULATION ? 24'd1770 : 24'd1770000,
    THRESHOLD_53   = FAST_SIMULATION ? 24'd1800 : 24'd1800000,
    THRESHOLD_54   = FAST_SIMULATION ? 24'd1830 : 24'd1830000,
    THRESHOLD_55   = FAST_SIMULATION ? 24'd1850 : 24'd1850000,
    THRESHOLD_56   = FAST_SIMULATION ? 24'd1880 : 24'd1880000,
    THRESHOLD_57   = FAST_SIMULATION ? 24'd1900 : 24'd1900000,
    THRESHOLD_58   = FAST_SIMULATION ? 24'd1930 : 24'd1930000,
    THRESHOLD_59   = FAST_SIMULATION ? 24'd1950 : 24'd1950000,
    THRESHOLD_60   = FAST_SIMULATION ? 24'd1980 : 24'd1980000,
    THRESHOLD_61   = FAST_SIMULATION ? 24'd2000 : 24'd2000000,
    THRESHOLD_62   = FAST_SIMULATION ? 24'd2010 : 24'd2010000,
    THRESHOLD_63   = FAST_SIMULATION ? 24'd2050 : 24'd2050000,
    THRESHOLD_64   = FAST_SIMULATION ? 24'd2080 : 24'd2080000,
    THRESHOLD_65   = FAST_SIMULATION ? 24'd2150 : 24'd2150000,
    THRESHOLD_66   = FAST_SIMULATION ? 24'd2200 : 24'd2200000,
    THRESHOLD_67   = FAST_SIMULATION ? 24'd2250 : 24'd2250000,
    THRESHOLD_68   = FAST_SIMULATION ? 24'd2270 : 24'd2270000,
    THRESHOLD_69   = FAST_SIMULATION ? 24'd2350 : 24'd2350000,
    THRESHOLD_70   = FAST_SIMULATION ? 24'd2400 : 24'd2400000,
    THRESHOLD_71   = FAST_SIMULATION ? 24'd2450 : 24'd2450000,
    THRESHOLD_72   = FAST_SIMULATION ? 24'd2500 : 24'd2500000,
    THRESHOLD_73   = FAST_SIMULATION ? 24'd2550 : 24'd2550000,
    THRESHOLD_74   = FAST_SIMULATION ? 24'd2580 : 24'd2580000,
    THRESHOLD_75   = FAST_SIMULATION ? 24'd2600 : 24'd2600000,
    THRESHOLD_76   = FAST_SIMULATION ? 24'd2650 : 24'd2650000,
    THRESHOLD_77   = FAST_SIMULATION ? 24'd2700 : 24'd2700000,
    THRESHOLD_78   = FAST_SIMULATION ? 24'd2750 : 24'd2750000,
    THRESHOLD_79   = FAST_SIMULATION ? 24'd2800 : 24'd2800000,
    THRESHOLD_80   = FAST_SIMULATION ? 24'd2850 : 24'd2850000,
    THRESHOLD_81   = FAST_SIMULATION ? 24'd2900 : 24'd2900000,
    THRESHOLD_82   = FAST_SIMULATION ? 24'd2950 : 24'd2950000,
    THRESHOLD_83   = FAST_SIMULATION ? 24'd3000 : 24'd3000000,
    THRESHOLD_84   = FAST_SIMULATION ? 24'd3030 : 24'd3030000,
    THRESHOLD_85   = FAST_SIMULATION ? 24'd3100 : 24'd3100000,
    THRESHOLD_86   = FAST_SIMULATION ? 24'd3150 : 24'd3150000,
    THRESHOLD_87   = FAST_SIMULATION ? 24'd3200 : 24'd3200000,
    THRESHOLD_88   = FAST_SIMULATION ? 24'd3250 : 24'd3250000,
    THRESHOLD_89   = FAST_SIMULATION ? 24'd3300 : 24'd3300000,
    THRESHOLD_90   = FAST_SIMULATION ? 24'd3350 : 24'd3350000,
    THRESHOLD_91   = FAST_SIMULATION ? 24'd3400 : 24'd3400000,
    THRESHOLD_92   = FAST_SIMULATION ? 24'd3520 : 24'd3520000,
    THRESHOLD_93   = FAST_SIMULATION ? 24'd3560 : 24'd3560000,
    THRESHOLD_94   = FAST_SIMULATION ? 24'd3720 : 24'd3720000,
    THRESHOLD_95   = FAST_SIMULATION ? 24'd3800 : 24'd3800000,
    THRESHOLD_96   = FAST_SIMULATION ? 24'd3920 : 24'd3920000,
    THRESHOLD_97   = FAST_SIMULATION ? 24'd4000 : 24'd4000000,
    THRESHOLD_98   = FAST_SIMULATION ? 24'd4040 : 24'd4040000,
    THRESHOLD_99  = FAST_SIMULATION ? 24'd4200 : 24'd4200000,
    THRESHOLD_100  = FAST_SIMULATION ? 24'd4320 : 24'd4320000,
    THRESHOLD_101  = FAST_SIMULATION ? 24'd4400 : 24'd4400000,
    THRESHOLD_102  = FAST_SIMULATION ? 24'd4520 : 24'd4520000,
    THRESHOLD_103  = FAST_SIMULATION ? 24'd4560 : 24'd4560000,
    THRESHOLD_104  = FAST_SIMULATION ? 24'd4720 : 24'd4720000,
    THRESHOLD_105  = FAST_SIMULATION ? 24'd4800 : 24'd4800000,
    THRESHOLD_106  = FAST_SIMULATION ? 24'd4920 : 24'd4920000,
    THRESHOLD_107  = FAST_SIMULATION ? 24'd5000 : 24'd5000000,
    THRESHOLD_108  = FAST_SIMULATION ? 24'd5120 : 24'd5120000,
    THRESHOLD_109  = FAST_SIMULATION ? 24'd5160 : 24'd5160000,
    THRESHOLD_110  = FAST_SIMULATION ? 24'd5200 : 24'd5200000,
    THRESHOLD_111  = FAST_SIMULATION ? 24'd5320 : 24'd5320000,
    THRESHOLD_112  = FAST_SIMULATION ? 24'd5400 : 24'd5400000,
    THRESHOLD_113  = FAST_SIMULATION ? 24'd5520 : 24'd5520000,
    THRESHOLD_114  = FAST_SIMULATION ? 24'd5600 : 24'd5600000,
    THRESHOLD_115  = FAST_SIMULATION ? 24'd5720 : 24'd5720000,
    THRESHOLD_116  = FAST_SIMULATION ? 24'd5800 : 24'd5800000,
    THRESHOLD_117  = FAST_SIMULATION ? 24'd5920 : 24'd5920000,
    THRESHOLD_118  = FAST_SIMULATION ? 24'd5960 : 24'd5960000,
    THRESHOLD_119  = FAST_SIMULATION ? 24'd6000 : 24'd6000000;
   
localparam [LOOPS_WIDTH-1:0]
    ROM_OFFSET_0  = 0,
    ROM_OFFSET_1  = 1,
    ROM_OFFSET_2  = 2,
    ROM_OFFSET_3  = 3,
    ROM_OFFSET_4  = 4,
    ROM_OFFSET_5  = 5,
    ROM_OFFSET_6  = 6,
    ROM_OFFSET_7  = 7,
    ROM_OFFSET_8  = 8,
    ROM_OFFSET_9  = 9,
    ROM_OFFSET_10 = 10,
    ROM_OFFSET_11 = 11,
    ROM_OFFSET_12 = 12,
    ROM_OFFSET_13 = 13,
    ROM_OFFSET_14 = 14,
    ROM_OFFSET_15 = 15,
    ROM_OFFSET_16 = 16,
    ROM_OFFSET_17 = 17,
    ROM_OFFSET_18 = 18,
    ROM_OFFSET_19 = 19,
    ROM_OFFSET_20 = 20,
    ROM_OFFSET_21 = 21,
    ROM_OFFSET_22 = 22,
    ROM_OFFSET_23 = 23,
    ROM_OFFSET_24 = 24,
    ROM_OFFSET_25 = 25,
    ROM_OFFSET_26 = 26,
    ROM_OFFSET_27 = 27,
    ROM_OFFSET_28 = 28,
    ROM_OFFSET_29 = 29,
    ROM_OFFSET_30 = 30,
    ROM_OFFSET_31 = 31,
    ROM_OFFSET_32 = 32,
    ROM_OFFSET_33 = 33,
    ROM_OFFSET_34 = 34,
    ROM_OFFSET_35 = 35,
    ROM_OFFSET_36 = 36,
    ROM_OFFSET_37 = 37,
    ROM_OFFSET_38 = 38,
    ROM_OFFSET_39 = 39,
    ROM_OFFSET_40 = 40,
    ROM_OFFSET_41 = 41,
    ROM_OFFSET_42 = 42,
    ROM_OFFSET_43 = 43,
    ROM_OFFSET_44 = 44,
    ROM_OFFSET_45 = 45,
    ROM_OFFSET_46 = 46,
    ROM_OFFSET_47 = 47,
    ROM_OFFSET_48 = 48,
    ROM_OFFSET_49 = 49,
    ROM_OFFSET_50 = 50,
    ROM_OFFSET_51 = 51,
    ROM_OFFSET_52 = 52,
    ROM_OFFSET_53 = 53,
    ROM_OFFSET_54 = 54,
    ROM_OFFSET_55 = 55,
    ROM_OFFSET_56 = 56,
    ROM_OFFSET_57 = 57,
    ROM_OFFSET_58 = 58,
    ROM_OFFSET_59 = 59,
    ROM_OFFSET_60 = 60,
    ROM_OFFSET_61 = 61,
    ROM_OFFSET_62 = 62,
    ROM_OFFSET_63 = 63,
    ROM_OFFSET_64 = 64,
    ROM_OFFSET_65 = 65,
    ROM_OFFSET_66 = 66,
    ROM_OFFSET_67 = 67,
    ROM_OFFSET_68 = 68,
    ROM_OFFSET_69 = 69,
    ROM_OFFSET_70 = 70,
    ROM_OFFSET_71 = 71,
    ROM_OFFSET_72 = 72,
    ROM_OFFSET_73 = 73,
    ROM_OFFSET_74 = 74,
    ROM_OFFSET_75 = 75,
    ROM_OFFSET_76 = 76,
    ROM_OFFSET_77 = 77,
    ROM_OFFSET_78 = 78,
    ROM_OFFSET_79 = 79,
    ROM_OFFSET_80 = 80,
    ROM_OFFSET_81 = 81,
    ROM_OFFSET_82 = 82,
    ROM_OFFSET_83 = 83,
    ROM_OFFSET_84 = 84,
    ROM_OFFSET_85 = 85,
    ROM_OFFSET_86 = 86,
    ROM_OFFSET_87 = 87,
    ROM_OFFSET_88 = 88,
    ROM_OFFSET_89 = 89,
    ROM_OFFSET_90 = 90,
    ROM_OFFSET_91 = 91,
    ROM_OFFSET_92 = 92,
    ROM_OFFSET_93 = 93,
    ROM_OFFSET_94 = 94,
    ROM_OFFSET_95 = 95,
    ROM_OFFSET_96 = 96,
    ROM_OFFSET_97 = 97,
    ROM_OFFSET_98 = 98,
    ROM_OFFSET_99 = 99,
    ROM_OFFSET_100 = 100,
    ROM_OFFSET_101 = 101,
    ROM_OFFSET_102 = 102,
    ROM_OFFSET_103 = 103,
    ROM_OFFSET_104 = 104,
    ROM_OFFSET_105 = 105,
    ROM_OFFSET_106 = 106,
    ROM_OFFSET_107 = 107,
    ROM_OFFSET_108 = 108,
    ROM_OFFSET_109 = 109,
    ROM_OFFSET_110 = 110,
    ROM_OFFSET_111 = 111,
    ROM_OFFSET_112 = 112,
    ROM_OFFSET_113 = 113,
    ROM_OFFSET_114 = 114,
    ROM_OFFSET_115 = 115,
    ROM_OFFSET_116 = 116,
    ROM_OFFSET_117 = 117,
    ROM_OFFSET_118 = 118,
    ROM_OFFSET_119 = 119;
   
// **** PERL SCIPT TO GENERATE END ****** //

reg                     current_state;
reg                     next_state;
reg                     inc_loops;
reg                     clr_loops;
reg                     set_valid;
reg [LOOPS_WIDTH-1:0]   loops;
reg [23:0]              threshold;
reg [POINTER_WIDTH-1:0] offset_ptr;
reg [POINTER_WIDTH-1:0] offset_ptr_r;
reg                     offset_ptr_valid_r;
reg [LOOPS_WIDTH-1:0]   range_r;
reg                     os;
reg                     os_r;
wire                    match;
wire                    loops_max;
   
// Simple FSM to sequentially compare against the predefined thresholds
always @ (posedge clock or posedge reset)
begin
    if (reset) begin
        current_state <= 1'b0;    
    end else begin
        current_state <= next_state;
    end  
end

always @ (*)
begin
    next_state = current_state;
    clr_loops = 1'b0;
    inc_loops = 1'b0;
    set_valid = 1'b0;
      
    case (current_state)
        0: begin
            clr_loops = 1'b1;
            if (measure_valid) begin
                next_state = 1'b1;
            end        
        end
        
        1: begin
            inc_loops = 1'b1;
            if (match | loops_max) begin
               set_valid = 1'b1;
               clr_loops = 1'b1;
               next_state = 1'b0;
            end                         
        end
    endcase  
end 

always @ (posedge clock or posedge reset)
begin
    if (reset) begin
       loops <= {LOOPS_WIDTH{1'b0}};
    end else begin
        if (clr_loops) begin
            loops <= 0;
        end else if (inc_loops) begin
            loops <= loops + {{{LOOPS_WIDTH-1}{1'b0}},1'b1};
        end 
    end
end 

// **** PERL SCIPT TO GENERATE BEGIN **** //
// 1.4 combinatorial
always @ (loops)
begin
    threshold  = THRESHOLD_119; 
    offset_ptr = ROM_OFFSET_119 * MIF_OFFSET; // was offset_ptr_r
    os         = 1'b0; // was or_r
    case (loops)                                                                                    
        0:    begin threshold = THRESHOLD_0;    offset_ptr = ROM_OFFSET_0   * MIF_OFFSET;   os = 1'b1; end
        1:    begin threshold = THRESHOLD_1;    offset_ptr = ROM_OFFSET_1   * MIF_OFFSET;   os = 1'b1; end
        2:    begin threshold = THRESHOLD_2;    offset_ptr = ROM_OFFSET_2   * MIF_OFFSET;   os = 1'b1; end
        3:    begin threshold = THRESHOLD_3;    offset_ptr = ROM_OFFSET_3   * MIF_OFFSET;   os = 1'b1; end
        4:    begin threshold = THRESHOLD_4;    offset_ptr = ROM_OFFSET_4   * MIF_OFFSET;   os = 1'b1; end
        5:    begin threshold = THRESHOLD_5;    offset_ptr = ROM_OFFSET_5   * MIF_OFFSET;   os = 1'b1; end
        6:    begin threshold = THRESHOLD_6;    offset_ptr = ROM_OFFSET_6   * MIF_OFFSET;   os = 1'b1; end
        7:    begin threshold = THRESHOLD_7;    offset_ptr = ROM_OFFSET_7   * MIF_OFFSET;   os = 1'b1; end
        8:    begin threshold = THRESHOLD_8;    offset_ptr = ROM_OFFSET_8   * MIF_OFFSET;   os = 1'b1; end
        9:    begin threshold = THRESHOLD_9;    offset_ptr = ROM_OFFSET_9   * MIF_OFFSET;   os = 1'b1; end
        10:   begin threshold = THRESHOLD_10;   offset_ptr = ROM_OFFSET_10  * MIF_OFFSET;   os = 1'b1; end
        11:   begin threshold = THRESHOLD_11;   offset_ptr = ROM_OFFSET_11  * MIF_OFFSET;   os = 1'b1; end
        12:   begin threshold = THRESHOLD_12;   offset_ptr = ROM_OFFSET_12  * MIF_OFFSET;   os = 1'b1; end
        13:   begin threshold = THRESHOLD_13;   offset_ptr = ROM_OFFSET_13  * MIF_OFFSET;   os = 1'b1; end
        14:   begin threshold = THRESHOLD_14;   offset_ptr = ROM_OFFSET_14  * MIF_OFFSET;   os = 1'b1; end
        15:   begin threshold = THRESHOLD_15;   offset_ptr = ROM_OFFSET_15  * MIF_OFFSET;   os = 1'b1; end
        16:   begin threshold = THRESHOLD_16;   offset_ptr = ROM_OFFSET_16  * MIF_OFFSET;   os = 1'b1; end
        17:   begin threshold = THRESHOLD_17;   offset_ptr = ROM_OFFSET_17  * MIF_OFFSET;   os = 1'b1; end
        18:   begin threshold = THRESHOLD_18;   offset_ptr = ROM_OFFSET_18  * MIF_OFFSET;   os = 1'b1; end
        19:   begin threshold = THRESHOLD_19;   offset_ptr = ROM_OFFSET_19  * MIF_OFFSET;   os = 1'b1; end
        20:   begin threshold = THRESHOLD_20;   offset_ptr = ROM_OFFSET_20  * MIF_OFFSET;   os = 1'b1; end
        21:   begin threshold = THRESHOLD_21;   offset_ptr = ROM_OFFSET_21  * MIF_OFFSET;   os = 1'b0; end
        22:   begin threshold = THRESHOLD_22;   offset_ptr = ROM_OFFSET_22  * MIF_OFFSET;   os = 1'b0; end
        23:   begin threshold = THRESHOLD_23;   offset_ptr = ROM_OFFSET_23  * MIF_OFFSET;   os = 1'b0; end
        24:   begin threshold = THRESHOLD_24;   offset_ptr = ROM_OFFSET_24  * MIF_OFFSET;   os = 1'b0; end
        25:   begin threshold = THRESHOLD_25;   offset_ptr = ROM_OFFSET_25  * MIF_OFFSET;   os = 1'b0; end
        26:   begin threshold = THRESHOLD_26;   offset_ptr = ROM_OFFSET_26  * MIF_OFFSET;   os = 1'b0; end
        27:   begin threshold = THRESHOLD_27;   offset_ptr = ROM_OFFSET_27  * MIF_OFFSET;   os = 1'b0; end
        28:   begin threshold = THRESHOLD_28;   offset_ptr = ROM_OFFSET_28  * MIF_OFFSET;   os = 1'b0; end
        29:   begin threshold = THRESHOLD_29;   offset_ptr = ROM_OFFSET_29  * MIF_OFFSET;   os = 1'b0; end
        30:   begin threshold = THRESHOLD_30;   offset_ptr = ROM_OFFSET_30  * MIF_OFFSET;   os = 1'b0; end
        31:   begin threshold = THRESHOLD_31;   offset_ptr = ROM_OFFSET_31  * MIF_OFFSET;   os = 1'b0; end
        32:   begin threshold = THRESHOLD_32;   offset_ptr = ROM_OFFSET_32  * MIF_OFFSET;   os = 1'b0; end
        33:   begin threshold = THRESHOLD_33;   offset_ptr = ROM_OFFSET_33  * MIF_OFFSET;   os = 1'b0; end
        34:   begin threshold = THRESHOLD_34;   offset_ptr = ROM_OFFSET_34  * MIF_OFFSET;   os = 1'b0; end
        35:   begin threshold = THRESHOLD_35;   offset_ptr = ROM_OFFSET_35  * MIF_OFFSET;   os = 1'b0; end
        36:   begin threshold = THRESHOLD_36;   offset_ptr = ROM_OFFSET_36  * MIF_OFFSET;   os = 1'b0; end
        37:   begin threshold = THRESHOLD_37;   offset_ptr = ROM_OFFSET_37  * MIF_OFFSET;   os = 1'b0; end
        38:   begin threshold = THRESHOLD_38;   offset_ptr = ROM_OFFSET_38  * MIF_OFFSET;   os = 1'b0; end
        39:   begin threshold = THRESHOLD_39;   offset_ptr = ROM_OFFSET_39  * MIF_OFFSET;   os = 1'b0; end
        40:   begin threshold = THRESHOLD_40;   offset_ptr = ROM_OFFSET_40  * MIF_OFFSET;   os = 1'b0; end
        41:   begin threshold = THRESHOLD_41;   offset_ptr = ROM_OFFSET_41  * MIF_OFFSET;   os = 1'b0; end
        42:   begin threshold = THRESHOLD_42;   offset_ptr = ROM_OFFSET_42  * MIF_OFFSET;   os = 1'b0; end
        43:   begin threshold = THRESHOLD_43;   offset_ptr = ROM_OFFSET_43  * MIF_OFFSET;   os = 1'b0; end
        44:   begin threshold = THRESHOLD_44;   offset_ptr = ROM_OFFSET_44  * MIF_OFFSET;   os = 1'b0; end
        45:   begin threshold = THRESHOLD_45;   offset_ptr = ROM_OFFSET_45  * MIF_OFFSET;   os = 1'b0; end
        46:   begin threshold = THRESHOLD_46;   offset_ptr = ROM_OFFSET_46  * MIF_OFFSET;   os = 1'b0; end
        47:   begin threshold = THRESHOLD_47;   offset_ptr = ROM_OFFSET_47  * MIF_OFFSET;   os = 1'b0; end
        48:   begin threshold = THRESHOLD_48;   offset_ptr = ROM_OFFSET_48  * MIF_OFFSET;   os = 1'b0; end
        49:   begin threshold = THRESHOLD_49;   offset_ptr = ROM_OFFSET_49  * MIF_OFFSET;   os = 1'b0; end
        50:   begin threshold = THRESHOLD_50;   offset_ptr = ROM_OFFSET_50  * MIF_OFFSET;   os = 1'b0; end
        51:   begin threshold = THRESHOLD_51;   offset_ptr = ROM_OFFSET_51  * MIF_OFFSET;   os = 1'b0; end
        52:   begin threshold = THRESHOLD_52;   offset_ptr = ROM_OFFSET_52  * MIF_OFFSET;   os = 1'b0; end
        53:   begin threshold = THRESHOLD_53;   offset_ptr = ROM_OFFSET_53  * MIF_OFFSET;   os = 1'b0; end
        54:   begin threshold = THRESHOLD_54;   offset_ptr = ROM_OFFSET_54  * MIF_OFFSET;   os = 1'b0; end
        55:   begin threshold = THRESHOLD_55;   offset_ptr = ROM_OFFSET_55  * MIF_OFFSET;   os = 1'b0; end
        56:   begin threshold = THRESHOLD_56;   offset_ptr = ROM_OFFSET_56  * MIF_OFFSET;   os = 1'b0; end
        57:   begin threshold = THRESHOLD_57;   offset_ptr = ROM_OFFSET_57  * MIF_OFFSET;   os = 1'b0; end
        58:   begin threshold = THRESHOLD_58;   offset_ptr = ROM_OFFSET_58  * MIF_OFFSET;   os = 1'b0; end
        59:   begin threshold = THRESHOLD_59;   offset_ptr = ROM_OFFSET_59  * MIF_OFFSET;   os = 1'b0; end
        60:   begin threshold = THRESHOLD_60;   offset_ptr = ROM_OFFSET_60  * MIF_OFFSET;   os = 1'b0; end
        61:   begin threshold = THRESHOLD_61;   offset_ptr = ROM_OFFSET_61  * MIF_OFFSET;   os = 1'b0; end
        62:   begin threshold = THRESHOLD_62;   offset_ptr = ROM_OFFSET_62  * MIF_OFFSET;   os = 1'b0; end
        63:   begin threshold = THRESHOLD_63;   offset_ptr = ROM_OFFSET_63  * MIF_OFFSET;   os = 1'b0; end
        64:   begin threshold = THRESHOLD_64;   offset_ptr = ROM_OFFSET_64  * MIF_OFFSET;   os = 1'b0; end
        65:   begin threshold = THRESHOLD_65;   offset_ptr = ROM_OFFSET_65  * MIF_OFFSET;   os = 1'b0; end
        66:   begin threshold = THRESHOLD_66;   offset_ptr = ROM_OFFSET_66  * MIF_OFFSET;   os = 1'b0; end
        67:   begin threshold = THRESHOLD_67;   offset_ptr = ROM_OFFSET_67  * MIF_OFFSET;   os = 1'b0; end
        68:   begin threshold = THRESHOLD_68;   offset_ptr = ROM_OFFSET_68  * MIF_OFFSET;   os = 1'b0; end
        69:   begin threshold = THRESHOLD_69;   offset_ptr = ROM_OFFSET_69  * MIF_OFFSET;   os = 1'b0; end
        70:   begin threshold = THRESHOLD_70;   offset_ptr = ROM_OFFSET_70  * MIF_OFFSET;   os = 1'b0; end
        71:   begin threshold = THRESHOLD_71;   offset_ptr = ROM_OFFSET_71  * MIF_OFFSET;   os = 1'b0; end
        72:   begin threshold = THRESHOLD_72;   offset_ptr = ROM_OFFSET_72  * MIF_OFFSET;   os = 1'b0; end
        73:   begin threshold = THRESHOLD_73;   offset_ptr = ROM_OFFSET_73  * MIF_OFFSET;   os = 1'b0; end
        74:   begin threshold = THRESHOLD_74;   offset_ptr = ROM_OFFSET_74  * MIF_OFFSET;   os = 1'b0; end
        75:   begin threshold = THRESHOLD_75;   offset_ptr = ROM_OFFSET_75  * MIF_OFFSET;   os = 1'b0; end
        76:   begin threshold = THRESHOLD_76;   offset_ptr = ROM_OFFSET_76  * MIF_OFFSET;   os = 1'b0; end
        77:   begin threshold = THRESHOLD_77;   offset_ptr = ROM_OFFSET_77  * MIF_OFFSET;   os = 1'b0; end
        78:   begin threshold = THRESHOLD_78;   offset_ptr = ROM_OFFSET_78  * MIF_OFFSET;   os = 1'b0; end
        79:   begin threshold = THRESHOLD_79;   offset_ptr = ROM_OFFSET_79  * MIF_OFFSET;   os = 1'b0; end
        80:   begin threshold = THRESHOLD_80;   offset_ptr = ROM_OFFSET_80  * MIF_OFFSET;   os = 1'b0; end
        81:   begin threshold = THRESHOLD_81;   offset_ptr = ROM_OFFSET_81  * MIF_OFFSET;   os = 1'b0; end
        82:   begin threshold = THRESHOLD_82;   offset_ptr = ROM_OFFSET_82  * MIF_OFFSET;   os = 1'b0; end
        83:   begin threshold = THRESHOLD_83;   offset_ptr = ROM_OFFSET_83  * MIF_OFFSET;   os = 1'b0; end
        84:   begin threshold = THRESHOLD_84;   offset_ptr = ROM_OFFSET_84  * MIF_OFFSET;   os = 1'b0; end
        85:   begin threshold = THRESHOLD_85;   offset_ptr = ROM_OFFSET_85  * MIF_OFFSET;   os = 1'b0; end
        86:   begin threshold = THRESHOLD_86;   offset_ptr = ROM_OFFSET_86  * MIF_OFFSET;   os = 1'b0; end
        87:   begin threshold = THRESHOLD_87;   offset_ptr = ROM_OFFSET_87  * MIF_OFFSET;   os = 1'b0; end
        88:   begin threshold = THRESHOLD_88;   offset_ptr = ROM_OFFSET_88  * MIF_OFFSET;   os = 1'b0; end
        89:   begin threshold = THRESHOLD_89;   offset_ptr = ROM_OFFSET_89  * MIF_OFFSET;   os = 1'b0; end
        90:   begin threshold = THRESHOLD_90;   offset_ptr = ROM_OFFSET_90  * MIF_OFFSET;   os = 1'b0; end
        91:   begin threshold = THRESHOLD_91;   offset_ptr = ROM_OFFSET_91  * MIF_OFFSET;   os = 1'b0; end
        92:   begin threshold = THRESHOLD_92;   offset_ptr = ROM_OFFSET_92  * MIF_OFFSET;   os = 1'b0; end
        93:   begin threshold = THRESHOLD_93;   offset_ptr = ROM_OFFSET_93  * MIF_OFFSET;   os = 1'b0; end
        94:   begin threshold = THRESHOLD_94;   offset_ptr = ROM_OFFSET_94  * MIF_OFFSET;   os = 1'b0; end
        95:   begin threshold = THRESHOLD_95;   offset_ptr = ROM_OFFSET_95  * MIF_OFFSET;   os = 1'b0; end
        96:   begin threshold = THRESHOLD_96;   offset_ptr = ROM_OFFSET_96  * MIF_OFFSET;   os = 1'b0; end
        97:   begin threshold = THRESHOLD_97;   offset_ptr = ROM_OFFSET_97  * MIF_OFFSET;   os = 1'b0; end
        98:   begin threshold = THRESHOLD_98;   offset_ptr = ROM_OFFSET_98  * MIF_OFFSET;   os = 1'b0; end
        99:   begin threshold = THRESHOLD_99;   offset_ptr = ROM_OFFSET_99  * MIF_OFFSET;   os = 1'b0; end
        100:  begin threshold = THRESHOLD_100;  offset_ptr = ROM_OFFSET_100 * MIF_OFFSET;   os = 1'b0; end
        101:  begin threshold = THRESHOLD_101;  offset_ptr = ROM_OFFSET_101 * MIF_OFFSET;   os = 1'b0; end
        102:  begin threshold = THRESHOLD_102;  offset_ptr = ROM_OFFSET_102 * MIF_OFFSET;   os = 1'b0; end
        103:  begin threshold = THRESHOLD_103;  offset_ptr = ROM_OFFSET_103 * MIF_OFFSET;   os = 1'b0; end
        104:  begin threshold = THRESHOLD_104;  offset_ptr = ROM_OFFSET_104 * MIF_OFFSET;   os = 1'b0; end
        105:  begin threshold = THRESHOLD_105;  offset_ptr = ROM_OFFSET_105 * MIF_OFFSET;   os = 1'b0; end
        106:  begin threshold = THRESHOLD_106;  offset_ptr = ROM_OFFSET_106 * MIF_OFFSET;   os = 1'b0; end
        107:  begin threshold = THRESHOLD_107;  offset_ptr = ROM_OFFSET_107 * MIF_OFFSET;   os = 1'b0; end
        108:  begin threshold = THRESHOLD_108;  offset_ptr = ROM_OFFSET_108 * MIF_OFFSET;   os = 1'b0; end
        109:  begin threshold = THRESHOLD_109;  offset_ptr = ROM_OFFSET_109 * MIF_OFFSET;   os = 1'b0; end
        110:  begin threshold = THRESHOLD_110;  offset_ptr = ROM_OFFSET_110 * MIF_OFFSET;   os = 1'b0; end
        111:  begin threshold = THRESHOLD_111;  offset_ptr = ROM_OFFSET_111 * MIF_OFFSET;   os = 1'b0; end
        112:  begin threshold = THRESHOLD_112;  offset_ptr = ROM_OFFSET_112 * MIF_OFFSET;   os = 1'b0; end
        113:  begin threshold = THRESHOLD_113;  offset_ptr = ROM_OFFSET_113 * MIF_OFFSET;   os = 1'b0; end
        114:  begin threshold = THRESHOLD_114;  offset_ptr = ROM_OFFSET_114 * MIF_OFFSET;   os = 1'b0; end
        115:  begin threshold = THRESHOLD_115;  offset_ptr = ROM_OFFSET_115 * MIF_OFFSET;   os = 1'b0; end
        116:  begin threshold = THRESHOLD_116;  offset_ptr = ROM_OFFSET_116 * MIF_OFFSET;   os = 1'b0; end
        117:  begin threshold = THRESHOLD_117;  offset_ptr = ROM_OFFSET_117 * MIF_OFFSET;   os = 1'b0; end
        118:  begin threshold = THRESHOLD_118;  offset_ptr = ROM_OFFSET_118 * MIF_OFFSET;   os = 1'b0; end
        119:  begin threshold = THRESHOLD_119;  offset_ptr = ROM_OFFSET_119 * MIF_OFFSET;   os = 1'b0; end
        default: begin 
		  threshold = THRESHOLD_119; offset_ptr = ROM_OFFSET_119 * MIF_OFFSET; os = 1'b0; end // <=600MHz
    endcase
end 
// **** PERL SCIPT TO GENERATE END ****** //

assign match = measure < threshold;
assign loops_max = loops == LOOPS_MAX;

// Outputs
always @ (posedge clock or posedge reset)
begin
    if (reset) begin
        offset_ptr_r <= ROM_OFFSET_119 * MIF_OFFSET;
        offset_ptr_valid_r <= 1'b0;
        os_r <= 1'b0;
        range_r <= {LOOPS_WIDTH{1'b0}};
    end else begin
        if (clr_valid | measure_valid) begin
            offset_ptr_valid_r <= 1'b0;
        end else if (set_valid) begin
            offset_ptr_r <= offset_ptr;
            offset_ptr_valid_r <= 1'b1;
            os_r <= os;
            range_r <= loops;
        end 
    end
end

assign offset_pointer = offset_ptr_r;
assign offset_pointer_valid = offset_ptr_valid_r;
assign oversampled = os_r;
assign range = range_r;

endmodule
