// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



//`define SEPARATE_RCFG_INTF_EN
`ifdef SEPARATE_RCFG_INTF_EN
    `define SEPARATE_RCFG_INTF 1
`else
    `define SEPARATE_RCFG_INTF 0
`endif

module s10_hdmi2_demo (
    //Clocks Inputs
    input        clk_fpga_b3_p, //LVDS - 100MHz (Programmable Si570) or SMA input
	 input              CLK_50_B2F,
    //User-IO
    input        cpu_resetn, 
    output [7:0] user_led_g, 
    input  		  user_pb,   
			    
    //FPGA Mezzanine Card (fmcb)
    //XCVR Interface///
    input        fmcb_gbtclk_m2c_p_0,             
//    input  [2:0] fmcb_dp_m2c_p,     
    output [3:0] fmcb_dp_c2m_p,     

    //Arria 10 LDVS 
    inout       fmcb_la_tx_p_11,
    inout       fmcb_la_rx_n_9,	
//    input       fmcb_la_rx_p_9,//rx 5v power
//    inout       fmcb_la_rx_p_8,     
//    inout       fmcb_la_rx_n_8, 
    input       fmcb_la_tx_p_10, 
    input       fmcb_la_tx_p_12,
    inout       fmcb_la_tx_n_12,
    inout       fmcb_la_rx_p_10
);

localparam SYMBOLS_PER_CLOCK = 2;
localparam SUPPORT_AUDIO = 1;
localparam SUPPORT_AUXILIARY = 1;
localparam SUPPORT_DEEP_COLOR = 1;

//valid value: 4, 6, 11, 0 (no configuration change due to schematics)
localparam BITEC_DAUGHTER_CARD_REV = 6;

//Invert polarity due to the polarity inversion on the Bitec HDMI FMC daugther card, you should disable this if the polarity inversion is not required
localparam TX_POLARITY_INVERSION = (BITEC_DAUGHTER_CARD_REV==6 || BITEC_DAUGHTER_CARD_REV==4)? 1 : 
                                   (BITEC_DAUGHTER_CARD_REV==11)? 0 : 0;
localparam RX_POLARITY_INVERSION = (BITEC_DAUGHTER_CARD_REV==6 || BITEC_DAUGHTER_CARD_REV==4)? 1 : 
                                   (BITEC_DAUGHTER_CARD_REV==11)? 0 : 0;
	  
wire clk100;
assign clk100 = clk_fpga_b3_p;

// -------------------------------------------------------------------------------
// Wait for device ready
// -------------------------------------------------------------------------------
wire device_ready;
device_init #(
  .CNTR_BITS  (16)
) device_init_inst (
  .clk (clk100),
  .init_done  (device_ready)
);

wire                                   sys_init;
wire                                   user_pb_0;
wire                                   mgmt_clk_reset_sync;
wire [SYMBOLS_PER_CLOCK*48-1:0]        hdmi_rx_data;
wire [SYMBOLS_PER_CLOCK-1:0]           hdmi_rx_vsync;
wire [SYMBOLS_PER_CLOCK-1:0]           hdmi_rx_hsync;
wire [SYMBOLS_PER_CLOCK-1:0]           hdmi_rx_de;
wire                                   hdmi_rx_vid_clk;
wire                                   hdmi_rx_ls_clk;
wire                                   hdmi_rx_iopll_locked;
wire                                   gxb_rx_ready;
wire [23:0]                            measure;
wire                                   measure_valid;
wire                                   rx_os;
wire [1:0]                             tx_os;
wire                                   tx_audio_de;
wire [255:0]                           tx_audio_data;
wire [47:0]                            tx_audio_info_ai;
wire [19:0]                            tx_audio_N;
wire [19:0]                            tx_audio_CTS;
wire [164:0]                           tx_audio_metadata;
wire [4:0]                             tx_audio_format;
wire [111:0]                           tx_info_avi;
wire [60:0]                            tx_info_vsi;
wire                                   rx_audio_de;
wire [255:0]                           rx_audio_data;
wire [47:0]                            rx_audio_info_ai;
wire [19:0]                            rx_audio_N;
wire [19:0]                            rx_audio_CTS;
wire [164:0]                           rx_audio_metadata;
wire [4:0]                             rx_audio_format;
wire [111:0]                           rx_info_avi;
wire [60:0]                            rx_info_vsi;
wire                                   TMDS_Bit_clock_Ratio;
wire                                   reset;
wire                                   hpd;
wire                                   rx_hpd;
wire                                   rx_hdmi_locked;
wire                                   rx_iopll_locked;
wire                                   rx_ready;
wire                                   tx_ready;
wire                                   tx_iopll_locked;
wire                                   tx_pll_locked;

wire                                   cpu_clk_reset_n;
wire                                   i2c_clk_reset_sync;
wire                                   tx_hpd_ack;
wire                                   tx_hpd_req;
wire  [23:0]                           reserve_writedata;



assign user_led_g[0] = ~tx_iopll_locked;
assign user_led_g[1] = ~tx_ready;
assign user_led_g[2] = ~tx_pll_locked;
assign user_led_g[3] = ~tx_os;
assign user_led_g[4] = ~tx_iopll_locked; 
assign user_led_g[5] = ~tx_ready; 
assign user_led_g[6] = ~tx_pll_locked; 
assign user_led_g[7] = ~|tx_os;
assign reset         = ~cpu_resetn | ~device_ready;
assign user_pb_0    = user_pb;


wire tmds_clk_in;
wire hdmi_clk_in;
wire mgmt_clk;
assign tmds_clk_in = fmcb_gbtclk_m2c_p_0;
assign hdmi_clk_in = tmds_clk_in;
assign mgmt_clk = clk100;


wire i2c_clk;
assign i2c_clk = clk100;

//
// HDMI RX video path and transceiver/PLL reconfig controller (master + slave)
//
wire [5:0]  tx_gcp;
wire [71:0] tx_aux_data;
wire        tx_aux_valid;
wire        tx_aux_sop;
wire        tx_aux_eop;
wire [5:0]  rx_gcp;
wire [71:0] rx_aux_data;
wire        rx_aux_valid;
wire        rx_aux_sop;
wire        rx_aux_eop;
wire        tx_aux_ready;

wire        rx_reconfig_mgmt_write;
wire        rx_reconfig_mgmt_read;
wire [12:0] rx_reconfig_mgmt_address;
wire [31:0] rx_reconfig_mgmt_writedata;
wire  [31:0] rx_reconfig_mgmt_readdata;
wire         rx_reconfig_mgmt_waitrequest;
wire [2:0]  gxb_rx_cal_busy_out;
wire        rx_reconfig_en;

wire [2:0] gxb_rx_reconfig_write;
wire [2:0] gxb_rx_reconfig_read;
wire [32:0] gxb_rx_reconfig_address;
wire [95:0] gxb_rx_reconfig_writedata;
wire [95:0] gxb_rx_reconfig_readdata;
wire [2:0] gxb_rx_reconfig_waitrequest;
wire [2:0]   gxb_rx_cal_busy_in;

wire        rx_mgmt_clk_reset;
wire [1:0]  color_depth_sync;

wire [7:0]  rx_edid_ram_address;
wire        rx_edid_ram_write;
wire        rx_edid_ram_read;
wire [7:0]  rx_edid_ram_readdata;
wire [7:0]  rx_edid_ram_writedata;
wire        rx_edid_ram_waitrequest;
wire        rx_edid_ram_access;

//
// Qsys system for TX reconfig & I2C master (TX DDC)
// Placeholder for VIP passthrough system
//
wire                            tx_rst_pll;
wire                            tx_rst_xcvr;
wire                            wd_reset;
wire                            hdmi_tx_vid_clk;
wire                            hdmi_tx_ls_clk;
wire [48*SYMBOLS_PER_CLOCK-1:0] hdmi_tx_data;
wire [1*SYMBOLS_PER_CLOCK-1:0]  hdmi_tx_vsync;
wire [1*SYMBOLS_PER_CLOCK-1:0]  hdmi_tx_hsync;
wire [1*SYMBOLS_PER_CLOCK-1:0]  hdmi_tx_de;
wire [10:0]                     tx_pll_reconfig_mgmt_address;
wire                            tx_pll_reconfig_mgmt_waitrequest;
wire                            tx_pll_reconfig_mgmt_write;
wire [31:0]                     tx_pll_reconfig_mgmt_writedata;
wire                            tx_pll_reconfig_mgmt_read;
wire [31:0]                     tx_pll_reconfig_mgmt_readdata;
wire [9:0]                      tx_iopll_reconfig_mgmt_address;
wire                            tx_iopll_reconfig_mgmt_waitrequest;
wire                            tx_iopll_reconfig_mgmt_write;
wire [31:0]                     tx_iopll_reconfig_mgmt_writedata;
wire                            tx_iopll_reconfig_mgmt_read;
wire [31:0]                     tx_iopll_reconfig_mgmt_readdata;

wire [10:0]                     tx_pma_rcfg_mgmt_address;
wire [12:0]                     tx_pma_rcfg_mgmt_translator_avalon_anti_slave_address;
wire                            tx_pma_rcfg_mgmt_write;
wire                            tx_pma_rcfg_mgmt_read;
wire [31:0]                     tx_pma_rcfg_mgmt_readdata;
wire [31:0]                     tx_pma_rcfg_mgmt_writedata;
wire                            tx_pma_rcfg_mgmt_waitrequest;
wire                            tx_reconfig_en;
wire [1:0]                      tx_pma_ch;

wire                            tx_measure_valid_ack;
wire                            tx_measure_valid_req;
wire  [23:0]                    tx_measure;
wire nios_tx_i2c_sda_in;
wire nios_tx_i2c_scl_in;
wire nios_tx_i2c_sda_oe;
wire nios_tx_i2c_scl_oe;

wire nios_ti_i2c_sda_in;
wire nios_ti_i2c_scl_in;
wire nios_ti_i2c_sda_oe;
wire nios_ti_i2c_scl_oe;
		
		
//wire [3*8*SYMBOLS_PER_CLOCK-1:0] cvi_data_in;
//wire [3*8*SYMBOLS_PER_CLOCK-1:0] cvo_data_out;
wire [47:0] cvo_data_out;
wire [1:0]  cvo_de;
wire [1:0]  cvo_vsync;
wire [1:0]  cvo_hsync;
wire [3:0]    gxb_tx_cal_busy_in;

assign cpu_clk_reset_n = ~(reset | wd_reset | sys_init);




wire clk_1485M;
    Test_Pattern Test_Pattern_inst (
        .alt_vip_cl_cvo_clocked_video_vid_clk       (hdmi_tx_vid_clk),       //   input,   width = 1, alt_vip_cl_cvo_clocked_video.vid_clk
        .alt_vip_cl_cvo_clocked_video_vid_data      (hdmi_tx_data),      //  output,  width = 30,                             .vid_data
        .alt_vip_cl_cvo_clocked_video_underflow     (),     //  output,   width = 1,                             .underflow
        .alt_vip_cl_cvo_clocked_video_vid_datavalid (hdmi_tx_de), //  output,   width = 1,                             .vid_datavalid
        .alt_vip_cl_cvo_clocked_video_vid_v_sync    (hdmi_tx_vsync),    //  output,   width = 1,                             .vid_v_sync
        .alt_vip_cl_cvo_clocked_video_vid_h_sync    (hdmi_tx_hsync),    //  output,   width = 1,                             .vid_h_sync
        .alt_vip_cl_cvo_clocked_video_vid_f         (),         //  output,   width = 1,                             .vid_f
        .alt_vip_cl_cvo_clocked_video_vid_h         (),         //  output,   width = 1,                             .vid_h
        .alt_vip_cl_cvo_clocked_video_vid_v         (),         //  output,   width = 1,                             .vid_v
        .clock_in_clk_clk                           (CLK_50_B2F),                           //   input,   width = 1,                 clock_in_clk.clk
       // .iopll_locked_export                        (),                        //  output,   width = 1,                 iopll_locked.export
//        .iopll_outclk0_clk                          (clk_1485M),                          //  output,   width = 1,                iopll_outclk0.clk
        .reset_in_reset_reset_n                     (cpu_resetn)                      //   input,   width = 1,               reset_in_reset.reset_n
    );



nios u_nios (
   /* I */ .clock_bridge_0_in_clk_clk (clk100),
   /* I */ .reset_bridge_0_in_reset_reset_n (cpu_clk_reset_n),
   /* I */ .tmds_bit_clock_ratio_pio_external_connection_export (TMDS_Bit_clock_Ratio), 
   /* I */ .measure_pio_external_connection_export (tx_measure),
   /* I */ .measure_valid_req_export (tx_measure_valid_req),
   /* O */ .measure_valid_ack_export (tx_measure_valid_ack),

   /* I */ .i2c_master_i2c_serial_sda_in(nios_tx_i2c_sda_in),
   /* I */ .i2c_master_i2c_serial_scl_in(nios_tx_i2c_scl_in),
   /* O */ .i2c_master_i2c_serial_sda_oe(nios_tx_i2c_sda_oe),
   /* O */ .i2c_master_i2c_serial_scl_oe(nios_tx_i2c_scl_oe),
   
   /* I */ .i2c_master_ti_i2c_serial_sda_in(nios_ti_i2c_sda_in),
   /* I */ .i2c_master_ti_i2c_serial_scl_in(nios_ti_i2c_scl_in),
   /* O */ .i2c_master_ti_i2c_serial_sda_oe(nios_ti_i2c_sda_oe),
   /* O */ .i2c_master_ti_i2c_serial_scl_oe(nios_ti_i2c_scl_oe),
  
   /* O */ .edid_ram_access_pio_external_connection_export(rx_edid_ram_access),
   /* O */ .edid_ram_slave_translator_address (rx_edid_ram_address),
   /* O */ .edid_ram_slave_translator_write (rx_edid_ram_write),
   /* O */ .edid_ram_slave_translator_read (rx_edid_ram_read),
   /* I */ .edid_ram_slave_translator_readdata (rx_edid_ram_readdata),
   /* O */ .edid_ram_slave_translator_writedata (rx_edid_ram_writedata),
   /* I */ .edid_ram_slave_translator_waitrequest (rx_edid_ram_waitrequest),

   /* I */ .color_depth_pio_external_connection_export (color_depth_sync),	
   /* I */ .tx_iopll_waitrequest_pio_external_connection_export (tx_iopll_reconfig_mgmt_waitrequest),
   /* I */ .tx_pll_waitrequest_pio_external_connection_export (tx_pll_reconfig_mgmt_waitrequest),
   /* I */ .tx_hpd_req_pio_external_connection_export (tx_hpd_req),
   /* O */ .tx_hpd_ack_pio_external_connection_export (tx_hpd_ack),					     
   /* I */ .tx_pll_rcfg_mgmt_translator_avalon_anti_slave_waitrequest (tx_pll_reconfig_mgmt_waitrequest),
   /* O */ .tx_pll_rcfg_mgmt_translator_avalon_anti_slave_writedata (tx_pll_reconfig_mgmt_writedata),
   /* O */ .tx_pll_rcfg_mgmt_translator_avalon_anti_slave_address (tx_pll_reconfig_mgmt_address),
   /* O */ .tx_pll_rcfg_mgmt_translator_avalon_anti_slave_write (tx_pll_reconfig_mgmt_write),
   /* O */ .tx_pll_rcfg_mgmt_translator_avalon_anti_slave_read (tx_pll_reconfig_mgmt_read),
   /* I */ .tx_pll_rcfg_mgmt_translator_avalon_anti_slave_readdata (tx_pll_reconfig_mgmt_readdata),					     
   /* O */ .tx_iopll_rcfg_mgmt_translator_avalon_anti_slave_writedata (tx_iopll_reconfig_mgmt_writedata),
   /* O */ .tx_iopll_rcfg_mgmt_translator_avalon_anti_slave_address (tx_iopll_reconfig_mgmt_address),
   /* O */ .tx_iopll_rcfg_mgmt_translator_avalon_anti_slave_write (tx_iopll_reconfig_mgmt_write),
   /* O */ .tx_iopll_rcfg_mgmt_translator_avalon_anti_slave_read (tx_iopll_reconfig_mgmt_read),
   /* I */ .tx_iopll_rcfg_mgmt_translator_avalon_anti_slave_readdata (tx_iopll_reconfig_mgmt_readdata),
   
   /* O */ .tx_pma_rcfg_mgmt_translator_avalon_anti_slave_address(tx_pma_rcfg_mgmt_translator_avalon_anti_slave_address),
   /* O */ .tx_pma_rcfg_mgmt_translator_avalon_anti_slave_write(tx_pma_rcfg_mgmt_write),
   /* O */ .tx_pma_rcfg_mgmt_translator_avalon_anti_slave_read(tx_pma_rcfg_mgmt_read),
   /* I */ .tx_pma_rcfg_mgmt_translator_avalon_anti_slave_readdata(tx_pma_rcfg_mgmt_readdata),
   /* O */ .tx_pma_rcfg_mgmt_translator_avalon_anti_slave_writedata(tx_pma_rcfg_mgmt_writedata),
   /* I */ .tx_pma_rcfg_mgmt_translator_avalon_anti_slave_waitrequest(tx_pma_rcfg_mgmt_waitrequest),
   /* I */ .tx_pma_waitrequest_pio_external_connection_export(tx_pma_rcfg_mgmt_waitrequest),
   /* I */ .tx_pma_cal_busy_pio_external_connection_export( {|gxb_tx_cal_busy_in} ),
   /* O */ .tx_pma_ch_export(tx_pma_ch),
   
   /* O */ .tx_rcfg_en_pio_external_connection_export(tx_reconfig_en),
   /* O */ .tx_os_pio_external_connection_export (tx_os),
   /* O */ .tx_rst_pll_pio_external_connection_export (tx_rst_pll),
   /* O */ .tx_rst_xcvr_pio_external_connection_export (tx_rst_xcvr),
   /* O */ .wd_timer_resetrequest_reset (wd_reset)
);

//wire [21:0] CTS_audio;
wire [3:0]  tx_serial_data;  

wire [3:0]    gxb_tx_reconfig_write;
wire [3:0]    gxb_tx_reconfig_read;
wire [43:0]   gxb_tx_reconfig_address;
wire [127:0]  gxb_tx_reconfig_writedata;
wire [127:0]  gxb_tx_reconfig_readdata;
wire [3:0]    gxb_tx_reconfig_waitrequest;
wire [3:0]    gxb_tx_cal_busy_out;

wire          tx_reset_pll_reconfig;
assign tx_reset_pll_reconfig = sys_init | wd_reset;

hdmi_tx_top #(
   .SUPPORT_DEEP_COLOR (SUPPORT_DEEP_COLOR),
   .SUPPORT_AUXILIARY (SUPPORT_AUXILIARY),
   .SYMBOLS_PER_CLOCK (SYMBOLS_PER_CLOCK),
   .SUPPORT_AUDIO (SUPPORT_AUDIO),
   .POLARITY_INVERSION (0),
   .BITEC_DAUGHTER_CARD_REV (BITEC_DAUGHTER_CARD_REV)
) u_hdmi_tx_top (
   /* I */ .mgmt_clk (mgmt_clk),
   /* I */ .reset (reset),
   /* I */ .hdmi_clk_in (hdmi_clk_in),
   /* O */ .vid_clk_out (hdmi_tx_vid_clk),
   /* O */ .ls_clk_out (hdmi_tx_ls_clk),		 
   /* O */ .iopll_locked (tx_iopll_locked),
   /* O */ .txpll_locked (tx_pll_locked),
   /* O */ .gxb_tx_ready (tx_ready),
   /* I */ .reset_xcvr (tx_rst_xcvr),  
   /* I */ .reset_pll (tx_rst_pll),  
   /* I */ //.reset_pll_reconfig (sys_init | wd_reset), 
   /* I */ .reset_pll_reconfig (tx_reset_pll_reconfig),
   /* I */ .sys_init (sys_init),
   
   /* O */ .nios_tx_i2c_sda_in(nios_tx_i2c_sda_in),
   /* O */ .nios_tx_i2c_scl_in(nios_tx_i2c_scl_in),
   /* I */ .nios_tx_i2c_sda_oe(nios_tx_i2c_sda_oe),
   /* I */ .nios_tx_i2c_scl_oe(nios_tx_i2c_scl_oe),
   
   /* O */ .nios_ti_i2c_sda_in(nios_ti_i2c_sda_in),
   /* O */ .nios_ti_i2c_scl_in(nios_ti_i2c_scl_in),
   /* I */ .nios_ti_i2c_sda_oe(nios_ti_i2c_sda_oe),
   /* I */ .nios_ti_i2c_scl_oe(nios_ti_i2c_scl_oe),

   /* O */ .tx_serial_data (tx_serial_data[3:0]),		 
   /* I */ //.gxb_reconfig_write (tx_gxb_reconfig_mgmt_write),
   /* I */ //.gxb_reconfig_read (tx_gxb_reconfig_mgmt_read),
   /* I */ //.gxb_reconfig_address (tx_gxb_reconfig_mgmt_address),
   /* I */ //.gxb_reconfig_writedata (tx_gxb_reconfig_mgmt_writedata),
   /* O */ //.gxb_reconfig_readdata (tx_gxb_reconfig_mgmt_readdata),
   /* O */ //.gxb_reconfig_waitrequest (tx_gxb_reconfig_mgmt_waitrequest),
   /* I */ .tx_pll_reconfig_write (tx_pll_reconfig_mgmt_write),
   /* I */ .tx_pll_reconfig_read (tx_pll_reconfig_mgmt_read),
   /* I */ .tx_pll_reconfig_address (tx_pll_reconfig_mgmt_address),
   /* I */ .tx_pll_reconfig_writedata (tx_pll_reconfig_mgmt_writedata),
   /* O */ .tx_pll_reconfig_readdata (tx_pll_reconfig_mgmt_readdata),
   /* O */ .tx_pll_reconfig_waitrequest (tx_pll_reconfig_mgmt_waitrequest),
   /* I */ .pll_reconfig_write (tx_iopll_reconfig_mgmt_write),
   /* I */ .pll_reconfig_read (tx_iopll_reconfig_mgmt_read),
   /* I */ .pll_reconfig_address (tx_iopll_reconfig_mgmt_address),
   /* I */ .pll_reconfig_writedata (tx_iopll_reconfig_mgmt_writedata),
   /* O */ .pll_reconfig_readdata (tx_iopll_reconfig_mgmt_readdata),
   /* O */ .pll_reconfig_waitrequest (tx_iopll_reconfig_mgmt_waitrequest),
   /* I */ .os (tx_os), 
   /* I */ .mode (1'b1), 
   /* I */ .ctrl ({SYMBOLS_PER_CLOCK{6'd0}}),					
   /* I */ .aux_data (tx_aux_data),
   /* I */ //.aux_valid (tx_aux_valid & ~ext_aux_block),
   /* I */ .aux_valid (tx_aux_valid),
   /* O */ .aux_ready (tx_aux_ready),
   /* I */ .aux_sop (tx_aux_sop),
   /* I */ .aux_eop (tx_aux_eop),
   /* I */ .gcp (6'h20),
           // Press user_pb[2], you will no longer see infoframes in the transmitted signal
           // Release user_pb[2], you will see continuous infoframes in the transmitted signal
   /* I */ .info_avi ({1'b0, 112'h10842802B1}),//112'h10842802B1 tx_info_avi
   /* I */ .info_vsi ({1'b0, 61'b0}),//61'b0 tx_info_vsi
   /* I */ //.audio_clk (),
   /* I */ .audio_de (tx_audio_de),
   /* I */ .audio_mute (1'b0),
   /* I */ .audio_CTS (tx_audio_CTS),
   /* I */ .audio_data (tx_audio_data), 
   /* I */ .audio_info_ai ({1'b1, tx_audio_info_ai}), 
   /* I */ .audio_N (tx_audio_N), 
   /* I */ .audio_metadata ({1'b1, tx_audio_metadata}),
   /* I */ .audio_format (tx_audio_format),
	
//   /* I */ .vid_data ({hdmi_tx_data[59:50],6'h0,hdmi_tx_data[49:40],6'h0,hdmi_tx_data[39:30],6'h0,hdmi_tx_data[29:20],6'h0,hdmi_tx_data[19:10],6'h0 ,hdmi_tx_data[9:0],6'h0}), // use tpg: hdmi_tx_data_from_cvo    

   /* I */ .vid_data (hdmi_tx_data), // use tpg: hdmi_tx_data_from_cvo    
   /* I */ .vid_de (hdmi_tx_de), // use tpg: hdmi_tx_de_from_cvo
   /* I */ .vid_hsync (hdmi_tx_hsync), // use tpg: hdmi_tx_hsync_from_cvo
   /* I */ .vid_vsync (hdmi_tx_vsync), // use tpg: hdmi_tx_vsync_from_cvo
	
   /* I */ .Scrambler_Enable (TMDS_Bit_clock_Ratio), 
   /* I */ .TMDS_Bit_clock_Ratio (TMDS_Bit_clock_Ratio),

   /* I */ .tx_hpd_ack (tx_hpd_ack), 
   /* O */ .tx_hpd_req (tx_hpd_req), 
   /* I */ .hdmi_tx_hpd_n (~fmcb_la_tx_p_12), 
   /* B */ .hdmi_tx_i2c_sda (fmcb_la_tx_n_12),
   /* B */ .hdmi_tx_i2c_scl (fmcb_la_rx_p_10),
  
   /* B */ .hdmi_ti_i2c_sda (fmcb_la_tx_p_11),
   /* B */ .hdmi_ti_i2c_scl (fmcb_la_rx_n_9),
  
   /* O */ .gxb_tx_cal_busy_out(gxb_tx_cal_busy_out),
   /* I */ .gxb_reconfig_write(gxb_tx_reconfig_write),
   /* I */ .gxb_reconfig_read(gxb_tx_reconfig_read),
   /* I */ .gxb_reconfig_address(gxb_tx_reconfig_address),
   /* I */ .gxb_reconfig_writedata(gxb_tx_reconfig_writedata),
   /* O */ .gxb_reconfig_readdata(gxb_tx_reconfig_readdata),
   /* O */ .gxb_reconfig_waitrequest(gxb_tx_reconfig_waitrequest),
   /* I */ .gxb_tx_cal_busy_in(gxb_tx_cal_busy_in),
   /* I */ .measure_valid_in(measure_valid),
   /* I */ .measure_in(measure),
   /* I */ .measure_valid_ack(tx_measure_valid_ack),
   /* O */ .measure_valid_req(tx_measure_valid_req),
   /* O */ .measure_out(tx_measure)

);




assign fmcb_dp_c2m_p = tx_serial_data;



assign tx_pma_rcfg_mgmt_address = tx_pma_rcfg_mgmt_translator_avalon_anti_slave_address[10:0];

//assign gxb_rx_reconfig_write = reconfig_write[2:0];
//assign gxb_rx_reconfig_read = reconfig_read[2:0];
//assign gxb_rx_reconfig_address = reconfig_address[32:0];
//assign gxb_rx_reconfig_writedata = reconfig_writedata[95:0];
//assign gxb_rx_cal_busy_in = rx_reconfig_cal_busy[2:0];
//
//assign gxb_tx_reconfig_write = reconfig_write[3:0];
//assign gxb_tx_reconfig_read = reconfig_read[3:0];
//assign gxb_tx_reconfig_address = reconfig_address[43:0];
//assign gxb_tx_reconfig_writedata = reconfig_writedata[127:0];




localparam LANES = 4;
localparam DPRIO_ADDRESS_WIDTH = 11;
localparam DPRIO_DATA_WIDTH = 32;
wire [LANES-1:0]                         reconfig_write;
wire [LANES-1:0]                         reconfig_read;
wire [(LANES*DPRIO_ADDRESS_WIDTH)-1:0]   reconfig_address;
wire [(LANES*DPRIO_DATA_WIDTH)-1:0]      reconfig_writedata;

xcvr_reconfig_arbiter #(
    .LANES(LANES),
    .DPRIO_ADDRESS_WIDTH(DPRIO_ADDRESS_WIDTH), 
    .DPRIO_DATA_WIDTH(DPRIO_DATA_WIDTH),
    .EXPIRED_COUNTER(32'h042C1D80)       // 700ms
) u_xcvr_reconfig_arbiter_inst(
    /* I */ .clk(mgmt_clk),            // This should be the same 100MHz clock driving the reconfig controller
    /* I */ .reset(reset),          // This should be the same reset driving the reconfig controller
    /* I */ .rx_rcfg_en(1'b0),
	 
    /* I */ .tx_rcfg_en(tx_reconfig_en), 
    /* I */ .tx_rcfg_ch(tx_pma_ch),

	 
    /* I */ .tx_reconfig_mgmt_write(tx_pma_rcfg_mgmt_write),
    /* I */ .tx_reconfig_mgmt_read(tx_pma_rcfg_mgmt_read),
    /* I */ .tx_reconfig_mgmt_address(tx_pma_rcfg_mgmt_address[10:0]),
    /* I */ .tx_reconfig_mgmt_writedata(tx_pma_rcfg_mgmt_writedata),
    /* O */ .tx_reconfig_mgmt_readdata(tx_pma_rcfg_mgmt_readdata),
    /* O */ .tx_reconfig_mgmt_waitrequest(tx_pma_rcfg_mgmt_waitrequest),
      
    /* O */ .reconfig_write(reconfig_write),
    /* O */ .reconfig_read(reconfig_read),
    /* O */ .reconfig_address(reconfig_address),
    /* O */ .reconfig_writedata(reconfig_writedata),

    /* I */ .tx_reconfig_readdata(gxb_tx_reconfig_readdata),
    /* I */ .tx_reconfig_waitrequest(gxb_tx_reconfig_waitrequest),

    /* I */ .tx_cal_busy(gxb_tx_cal_busy_out),
    /* O */ .tx_reconfig_cal_busy(gxb_tx_cal_busy_in)
);

assign tx_pma_rcfg_mgmt_address = tx_pma_rcfg_mgmt_translator_avalon_anti_slave_address[10:0];

assign gxb_tx_reconfig_write = reconfig_write[3:0];
assign gxb_tx_reconfig_read = reconfig_read[3:0];
assign gxb_tx_reconfig_address = reconfig_address[43:0];
assign gxb_tx_reconfig_writedata = reconfig_writedata[127:0];

        
endmodule



