// ============================================================================
// Copyright (c) 2019 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// ============================================================================
//
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Wed Jun 26 16:32:24 2019
// ============================================================================

`define ENABLE_DDR4A
`define ENABLE_DDR4B
`define ENABLE_FMC
`define ENABLE_FMCP
`define ENABLE_PCIE
//`define ENABLE_QSFP28A
//`define ENABLE_QSFP28B
//`define ENABLE_HPS

module S10C_top(

      ///////// CLOCK /////////
      input              CLK_100_B3I,
      input              CLK_50_B2F,
      input              CLK_50_B2L,
      input              CLK_50_B3C,
      input              CLK_50_B3I,
      input              LVDS_REFCLK_p,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON,

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 1: 0]   LED, //LED is Low-Active

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output             DDR4A_CK,
      output             DDR4A_CK_n,
      output   [ 1: 0]   DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output   [ 1: 0]   DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output   [ 1: 0]   DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 1: 0]   DDR4B_BG,
      output             DDR4B_CK,
      output             DDR4B_CK_n,
      output   [ 1: 0]   DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output   [ 1: 0]   DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output   [ 1: 0]   DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

      ///////// SI5341A /////////
      inout              SI5341A_I2C_SCL,
      inout              SI5341A_I2C_SDA,
      output             SI5341A_OE_n,
      output             SI5341A_RST_n,

`ifdef ENABLE_FMC
      ///////// FMC /////////
      input    [ 1: 0]   S10C_KEY,
      input    [ 1: 0]   S10C_SW,
      output   [ 3: 0]   S10C_LED,
      inout    [35: 0]   FMC_GPIO_D,
`endif /*ENABLE_FMC*/

`ifdef ENABLE_FMCP
      ///////// FMCP /////////
      input              FMCP_REFCLK_p,
      inout              Si5340A_I2C_SCL,                    // FMCP_SCL
      inout              Si5340A_I2C_SDA,                    // FMCP_SDA
      output             SI5340A_OE_n,
      output             SI5340A_RST_n,

`ifdef ENABLE_PCIE
      input              DG_PCIe_RST_n,
      output             DG_PEWAKE_n_FMCP,
      input              DG_PM_S0_EN_FMCP,
      input              DG_PM_S3_EN_FMCP,
      input              PCIE_REFCLK_p,
      input    [ 3: 0]   PCIE_RX_p,
      output   [ 3: 0]   PCIE_TX_p,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFP28A
      input              QSFP28A_INTERRUPT_n,
      output             QSFP28A_LP_MODE,
      input              QSFP28A_MOD_PRS_n,
      output             QSFP28A_MOD_SEL_n,
      output             QSFP28A_RST_n,
      inout              QSFP28A_SCL,
      inout              QSFP28A_SDA,
      input              QSFP28A_REFCLK_p,
      input    [ 3: 0]   QSFP28A_RX_p,
      output   [ 3: 0]   QSFP28A_TX_p,
`endif /*ENABLE_QSFP28A*/

`ifdef ENABLE_QSFP28B
      input              QSFP28B_INTERRUPT_n,
      output             QSFP28B_LP_MODE,
      input              QSFP28B_MOD_PRS_n,
      output             QSFP28B_MOD_SEL_n,
      output             QSFP28B_RST_n,
      inout              QSFP28B_SCL,
      inout              QSFP28B_SDA,
      input              QSFP28B_REFCLK_p,
      input    [ 3: 0]   QSFP28B_RX_p,
      output   [ 3: 0]   QSFP28B_TX_p,
`endif /*ENABLE_QSFP28B*/
`endif /*ENABLE_FMCP*/

`ifdef ENABLE_HPS
      ///////// HPS /////////
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      inout    [ 3: 0]   HPS_GPIO,
      inout              HPS_I2C_SCL,
      inout              HPS_I2C_SDA,
      inout              HPS_KEY,
      inout              HPS_LED,
      input              HPS_OSC_CLK,
      output             HPS_SD_CLK,
      inout              HPS_SD_CMD,
      inout    [ 3: 0]   HPS_SD_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
      input              HPS_USB_CLK,
      inout    [ 7: 0]   HPS_USB_DATA,
      input              HPS_USB_DIR,
      input              HPS_USB_NXT,
      output             HPS_USB_STP,
`endif /*ENABLE_HPS*/

      ///////// S10MAX10 /////////
      output             INFO_SPI_SCLK,
      output             INFO_SPI_CS_n,
      output             INFO_SPI_MOSI,
      input              INFO_SPI_MISO


);

wire [5:0]   pio_led;
wire [3:0]   pio_button;

wire [66:0] hip_ctrl_test_in;

assign hip_ctrl_test_in = 67'h000000A8;
assign PCIE_WAKE_n = 1'b1;

assign {LED[1: 0], S10C_LED[3: 0]} = ~pio_led[5: 0]; // led low-active
assign pio_button = ~{BUTTON[1: 0], S10C_KEY[1: 0]}; // button low-active

//////////////////////
// PCIE RESET
wire             any_rstn;
reg              any_rstn_r /* synthesis ALTERA_ATTRIBUTE = "SUPPRESS_DA_RULE_INTERNAL=R102"  */;
reg              any_rstn_rr /* synthesis ALTERA_ATTRIBUTE = "SUPPRESS_DA_RULE_INTERNAL=R102"  */;
wire             ninit_done;

ResetRelease ResetRelease_inst (
    .ninit_done (ninit_done)  //  output,  width = 1, ninit_done.ninit_done
);

assign any_rstn = DG_PCIe_RST_n && CPU_RESET_n && !ninit_done;


//reset Synchronizer
always @(posedge CLK_50_B2F or negedge any_rstn) begin
    if (any_rstn == 0) begin
        any_rstn_r <= 0;
        any_rstn_rr <= 0;
    end else begin
        any_rstn_r <= 1;
        any_rstn_rr <= any_rstn_r;
    end
end

wire        ddr4a_local_cal_success;
wire        ddr4a_local_cal_fail;
wire	[1:0]	ddr4a_status;
assign ddr4a_status = {ddr4a_local_cal_fail, ddr4a_local_cal_success};

wire        ddr4b_local_cal_success;
wire        ddr4b_local_cal_fail;
wire	[1:0]	ddr4b_status;
assign ddr4b_status = {ddr4b_local_cal_fail, ddr4b_local_cal_success};

ep_g3x8_avmmdma256_integrated u0 (
    .hip_ctrl_test_in                       (hip_ctrl_test_in),
    //   input,  width = 67,                               .test_in
    .pcie_rstn_npor                         (any_rstn_rr),
    //   input,   width = 1,                      pcie_rstn.npor
    .pcie_rstn_pin_perst                    (DG_PCIe_RST_n),
    //   input,   width = 1,                               .pin_perst
    .ninit_done_ninit_done                  (ninit_done),
    //   input,   width = 1,                               ninit_done.ninit_done
    .pio_button_external_connection_export  (pio_button),
    //   input,   width = 2, pio_button_external_connection.export
    .pio_led_external_connection_export     (pio_led),
    //  output,   width = 4,    pio_led_external_connection.export
    .refclk_clk                             (PCIE_REFCLK_p),
    //   input,   width = 1,                         refclk.clk
    .xcvr_rx_in0                            (PCIE_RX_p[0]),
    //   input,   width = 1,                           xcvr.rx_in0
    .xcvr_rx_in1                            (PCIE_RX_p[1]),
    //   input,   width = 1,                               .rx_in1
    .xcvr_rx_in2                            (PCIE_RX_p[2]),
    //   input,   width = 1,                               .rx_in2
    .xcvr_rx_in3                            (PCIE_RX_p[3]),
    //   input,   width = 1,                               .rx_in3
    .xcvr_tx_out0                           (PCIE_TX_p[0]),
    //  output,   width = 1,                               .tx_out0
    .xcvr_tx_out1                           (PCIE_TX_p[1]),
    //  output,   width = 1,                               .tx_out1
    .xcvr_tx_out2                           (PCIE_TX_p[2]),
    //  output,   width = 1,                               .tx_out2
    .xcvr_tx_out3                           (PCIE_TX_p[3]),
    //  output,   width = 1,                               .tx_out3

    //DDR4A
    .emif_s10_ddr4a_ctrl_ecc_user_interrupt_0_ctrl_ecc_user_interrupt (),
    //  output,   width = 1, emif_s10_ddr4a_ctrl_ecc_user_interrupt_0.ctrl_ecc_user_interrupt
    .emif_s10_ddr4a_local_reset_req_local_reset_req                   (1'b1),
    //   input,   width = 1,           emif_s10_ddr4a_local_reset_req.local_reset_req
    .emif_s10_ddr4a_local_reset_status_local_reset_done               (),
    //  output,   width = 1,        emif_s10_ddr4a_local_reset_status.local_reset_done
    .emif_s10_ddr4a_mem_mem_ck                                        (DDR4A_CK),
    //  output,   width = 1,                       emif_s10_ddr4a_mem.mem_ck
    .emif_s10_ddr4a_mem_mem_ck_n                                      (DDR4A_CK_n),
    //  output,   width = 1,                                         .mem_ck_n
    .emif_s10_ddr4a_mem_mem_a                                         (DDR4A_A),
    //  output,  width = 17,                                         .mem_a
    .emif_s10_ddr4a_mem_mem_act_n                                     (DDR4A_ACT_n),
    //  output,   width = 1,                                         .mem_act_n
    .emif_s10_ddr4a_mem_mem_ba                                        (DDR4A_BA),
    //  output,   width = 2,                                         .mem_ba
    .emif_s10_ddr4a_mem_mem_bg                                        (DDR4A_BG),
    //  output,   width = 2,                                         .mem_bg
    .emif_s10_ddr4a_mem_mem_cke                                       (DDR4A_CKE),
    //  output,   width = 1,                                         .mem_cke
    .emif_s10_ddr4a_mem_mem_cs_n                                      (DDR4A_CS_n),
    //  output,   width = 1,                                         .mem_cs_n
    .emif_s10_ddr4a_mem_mem_odt                                       (DDR4A_ODT),
    //  output,   width = 1,                                         .mem_odt
    .emif_s10_ddr4a_mem_mem_reset_n                                   (DDR4A_RESET_n),
    //  output,   width = 1,                                         .mem_reset_n
    .emif_s10_ddr4a_mem_mem_par                                       (DDR4A_PAR),
    //  output,   width = 1,                                         .mem_par
    .emif_s10_ddr4a_mem_mem_alert_n                                   (DDR4A_ALERT_n),
    //   input,   width = 1,                                         .mem_alert_n
    .emif_s10_ddr4a_mem_mem_dqs                                       (DDR4A_DQS),
    //   inout,   width = 9,                                         .mem_dqs
    .emif_s10_ddr4a_mem_mem_dqs_n                                     (DDR4A_DQS_n),
    //   inout,   width = 9,                                         .mem_dqs_n
    .emif_s10_ddr4a_mem_mem_dq                                        (DDR4A_DQ),
    //   inout,  width = 72,                                         .mem_dq
    .emif_s10_ddr4a_mem_mem_dbi_n                                     (DDR4A_DBI_n),
    //   inout,   width = 9,                                         .mem_dbi_n
    .emif_s10_ddr4a_oct_oct_rzqin                                     (DDR4A_RZQ),
    //   input,   width = 1,                       emif_s10_ddr4a_oct.oct_rzqin
    .emif_s10_ddr4a_pll_ref_clk_clk                                   (DDR4A_REFCLK_p),
    //   input,   width = 1,               emif_s10_ddr4a_pll_ref_clk.clk
    .emif_s10_ddr4a_status_local_cal_success                          (ddr4a_local_cal_success),
    //  output,   width = 1,                    emif_s10_ddr4a_status.local_cal_success
    .emif_s10_ddr4a_status_local_cal_fail                             (ddr4a_local_cal_fail),
    //  output,   width = 1,                                         .local_cal_fail

    //DDR4B
    .emif_s10_ddr4b_ctrl_ecc_user_interrupt_0_ctrl_ecc_user_interrupt (),
    //  output,   width = 1, emif_s10_ddr4b_ctrl_ecc_user_interrupt_0.ctrl_ecc_user_interrupt
    .emif_s10_ddr4b_local_reset_req_local_reset_req                   (1'b1),
    //   input,   width = 1,           emif_s10_ddr4b_local_reset_req.local_reset_req
    .emif_s10_ddr4b_local_reset_status_local_reset_done               (),
    //  output,   width = 1,        emif_s10_ddr4b_local_reset_status.local_reset_done
    .emif_s10_ddr4b_mem_mem_ck                                        (DDR4B_CK),
    //  output,   width = 1,                       emif_s10_ddr4b_mem.mem_ck
    .emif_s10_ddr4b_mem_mem_ck_n                                      (DDR4B_CK_n),
    //  output,   width = 1,                                         .mem_ck_n
    .emif_s10_ddr4b_mem_mem_a                                         (DDR4B_A),
    //  output,  width = 17,                                         .mem_a
    .emif_s10_ddr4b_mem_mem_act_n                                     (DDR4B_ACT_n),
    //  output,   width = 1,                                         .mem_act_n
    .emif_s10_ddr4b_mem_mem_ba                                        (DDR4B_BA),
    //  output,   width = 2,                                         .mem_ba
    .emif_s10_ddr4b_mem_mem_bg                                        (DDR4B_BG),
    //  output,   width = 2,                                         .mem_bg
    .emif_s10_ddr4b_mem_mem_cke                                       (DDR4B_CKE),
    //  output,   width = 1,                                         .mem_cke
    .emif_s10_ddr4b_mem_mem_cs_n                                      (DDR4B_CS_n),
    //  output,   width = 1,                                         .mem_cs_n
    .emif_s10_ddr4b_mem_mem_odt                                       (DDR4B_ODT),
    //  output,   width = 1,                                         .mem_odt
    .emif_s10_ddr4b_mem_mem_reset_n                                   (DDR4B_RESET_n),
    //  output,   width = 1,                                         .mem_reset_n
    .emif_s10_ddr4b_mem_mem_par                                       (DDR4B_PAR),
    //  output,   width = 1,                                         .mem_par
    .emif_s10_ddr4b_mem_mem_alert_n                                   (DDR4B_ALERT_n),
    //   input,   width = 1,                                         .mem_alert_n
    .emif_s10_ddr4b_mem_mem_dqs                                       (DDR4B_DQS),
    //   inout,   width = 9,                                         .mem_dqs
    .emif_s10_ddr4b_mem_mem_dqs_n                                     (DDR4B_DQS_n),
    //   inout,   width = 9,                                         .mem_dqs_n
    .emif_s10_ddr4b_mem_mem_dq                                        (DDR4B_DQ),
    //   inout,  width = 72,                                         .mem_dq
    .emif_s10_ddr4b_mem_mem_dbi_n                                     (DDR4B_DBI_n),
    //   inout,   width = 9,                                         .mem_dbi_n
    .emif_s10_ddr4b_oct_oct_rzqin                                     (DDR4B_RZQ),
    //   input,   width = 1,                       emif_s10_ddr4b_oct.oct_rzqin
    .emif_s10_ddr4b_pll_ref_clk_clk                                   (DDR4B_REFCLK_p),
    //   input,   width = 1,               emif_s10_ddr4b_pll_ref_clk.clk
    .emif_s10_ddr4b_status_local_cal_success                          (ddr4b_local_cal_success),
    //  output,   width = 1,                    emif_s10_ddr4b_status.local_cal_success
    .emif_s10_ddr4b_status_local_cal_fail                             (ddr4b_local_cal_fail)
    //  output,   width = 1,                                         .local_cal_fail

);

assign SI5341A_RST_n = 1'b1;
assign SI5341A_OE_n = 1'b0;

assign DG_PEWAKE_n_FMCP = 1'b1;

endmodule
