// (C) 2001-2018 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ps / 1ps
module basic_avl_tb_top_nc ();
    reg          clk_ref;
    reg          clk_status;
    reg          csr_rst_n;
    reg          l2_tx_endofpacket;
    reg          l2_tx_startofpacket;
    reg          l2_tx_error;
    reg          l2_tx_valid;
    reg          reconfig_clk;
    reg          reconfig_read;
    reg          reconfig_reset;
    reg          reconfig_write;
    reg          reset_status;
    reg          rx_rst_n;
    reg          status_read;
    reg          status_write;
    reg          tx_rst_n;
    reg  [12:0]  reconfig_address;
    reg  [127:0] l2_tx_data;
    reg  [63:0]  l2_tx_preamble = 64'hfb555555555555d5;
    reg  [15:0]  status_addr;
    reg  [31:0]  reconfig_writedata;
    reg  [31:0]  status_writedata;
    reg  [3:0]   l2_tx_empty;


// localparam fc_width = 1 ;



    wire [3:0]   rx_serial;
    wire         tx_serial_clk;
    wire         tx_pll_locked;
    wire         clk_txmac;
    wire         clk_rxmac;
    wire         l2_rx_endofpacket;
    wire         l2_rx_startofpacket;
    wire         l2_rx_valid;
    wire         l2_rxstatus_valid;
    wire         l2_tx_ready;
    wire         l2_txstatus_valid;
    wire         reconfig_waitrequest;
    wire         rx_am_lock;
    wire         rx_block_lock;
    wire         rx_pcs_ready;
    wire         status_readdata_valid;
    wire         status_waitrequest;
    wire         tx_lanes_stable;
    wire [127:0] l2_rx_data;
    wire [63:0]  l2_rx_preamble;
    wire [31:0]  reconfig_readdata;
    wire [31:0]  status_readdata;
    wire [39:0]  l2_rxstatus_data;
    wire [39:0]  l2_txstatus_data;
    wire [3:0]   l2_rx_empty;
    wire [3:0]   tx_serial;
    wire [5:0]   l2_rx_error;
    wire [6:0]   l2_txstatus_error;
    wire         remote_fault_status;
    wire         local_fault_status;




    // -------------------------------
    // Ready Latency
    reg l2_tx_ready_1, l2_tx_ready_2, l2_tx_ready_3;
    wire tx_ready;
   
    always @(posedge clk_txmac) begin
        l2_tx_ready_1 <= l2_tx_ready;
        l2_tx_ready_2 <= l2_tx_ready_1;
        l2_tx_ready_3 <= l2_tx_ready_2;
    end
    

    assign tx_ready = l2_tx_ready; 
    // -------------------------------

    initial begin
        reconfig_writedata  = 0;
        reconfig_read       = 0;
        reconfig_reset      = 1;
        reconfig_write      = 0;
        reconfig_address    = 0;
        repeat (50) @(negedge clk_ref);
        reconfig_reset      = 0;
    end

    initial begin
        status_writedata    = 0;
        status_read         = 0;
        status_write        = 0;
        status_addr         = 0;
// reset_status pin is deprecated now and connect it to 0
        reset_status        = 0;
    end

    initial begin
        // $vcdpluson;
        csr_rst_n           = 0;
        l2_tx_data          = 0;
        l2_tx_endofpacket   = 0;
        l2_tx_startofpacket = 0;
        l2_tx_error         = 0;
        l2_tx_valid         = 0;
        l2_tx_empty         = 0;
        rx_rst_n            = 0;
        tx_rst_n            = 0;
        repeat (50) @(negedge clk_ref);

        while (tx_pll_locked !== 1) @(negedge clk_ref);


        repeat (20) @(negedge clk_ref);

        csr_rst_n           = 1;
        rx_rst_n            = 1;
        tx_rst_n            = 1;
        repeat (10) @(negedge clk_ref);

        $display ("Waiting for RX alignment");
        while (rx_pcs_ready !== 1) @(negedge clk_status);
        $display ("RX deskew locked");
        $display ("RX lane aligmnent locked");

        repeat (1000) @(posedge clk_txmac);
        $display ("TX enabled");
        send_packets_50g_avl(10);

        $display("**");
        $display("** Testbench complete.");
        $display("**");
        $display("*****************************************");

        repeat (10) @(negedge clk_ref);
        $finish();
    end

    task send_packets_50g_avl;
        input  [31:0]  number_of_packets;
        integer i,j,k;
        integer data_word;
        begin
            fork
                for (i = 1; i <= number_of_packets; i = i +1) begin
                    @(posedge clk_txmac);

                    wait_for_ready_avl;  
                    $display("** Sending Packet %d...",i);
                    l2_tx_data          = 128'hABE42339_F0001E42_339F0100_00000000;
                    l2_tx_startofpacket = 1'b1;
                    l2_tx_valid         = 1'b1;
                    wait_for_ready_avl;

                    for (k = 1; k <= 3; k = k +1) begin
                        data_word = i + (k << 8) + (k << 16) + (k << 24);
                        l2_tx_data          = {{3{data_word}}, i};
                        l2_tx_startofpacket = 1'b0;
                        l2_tx_valid         = 1'b1;
                        l2_tx_endofpacket = (k == 3) ? 1'b1 : 1'b0;
                        l2_tx_empty = (k == 3) ? 'd4 : 'd0;
                        wait_for_ready_avl;
                    end
                
                    l2_tx_data          = 0;
                    l2_tx_endofpacket   = 0;
                    l2_tx_startofpacket = 0;
                    l2_tx_valid         = 0;
                    l2_tx_empty         = 0;
                end
            
                for (j = 1; j <= number_of_packets; j = j+1) begin
                    while ( !(l2_rx_valid && !l2_rx_startofpacket && !l2_rx_endofpacket)) @(posedge clk_rxmac);
                    $display("** Received Packet %d...",l2_rx_data[31:0]);
                    @(posedge clk_rxmac);
                    while (!l2_rx_startofpacket && (j != number_of_packets)) @(posedge clk_rxmac);
                    @(posedge clk_rxmac);
                end
            join
        end
    endtask

    task wait_for_ready_avl;
        #1
        if(!tx_ready) begin
            while(!tx_ready) @(posedge clk_txmac);
        end
        else begin
            @(posedge clk_txmac);
        end
    endtask // wait_for_ready_avl
   
   

    localparam ref_prd = 1552; // 644.53125 MHz
    initial begin
        clk_ref = 1;
        forever #(ref_prd/2) clk_ref = ~clk_ref;
    end

    localparam avmm_prd = 10000; // 100 MHz
    initial begin
        clk_status = 1;
        forever #(avmm_prd/2) clk_status = ~clk_status;
    end

    initial begin
        reconfig_clk = 1;
        forever #(avmm_prd/2) reconfig_clk = ~reconfig_clk;
    end

    assign rx_serial = tx_serial;
//    ex_40g_s10 dut (.*);

    ex_40g_s10 dut (
	//.loop_done		( source_reset[7]      ),
        .clk_ref                (clk_ref),
        .csr_rst_n              (csr_rst_n),
        .tx_rst_n               (tx_rst_n),
        .rx_rst_n               (rx_rst_n),
        .tx_serial_clk          (tx_serial_clk),
        .tx_pll_locked          (tx_pll_locked),
        .clk_status             (clk_status),
//        .reset_status           (1'b0),


        .rx_serial              (rx_serial),
        .tx_serial              (tx_serial),

        .status_write           (status_write),
        .status_read            (status_read),
        .status_addr            (status_addr),
        .status_writedata       (status_writedata),
        .status_readdata        (status_readdata),
        .status_readdata_valid  (status_readdata_valid),
        .status_waitrequest     (status_waitrequest), // NOT USE DUT WAITREQUEST?

        .clk_txmac              (clk_txmac),
        .l2_tx_startofpacket    (l2_tx_startofpacket),
        .l2_tx_endofpacket      (l2_tx_endofpacket),
        .l2_tx_valid            (l2_tx_valid),
        .l2_tx_ready            (l2_tx_ready),
        .l2_tx_empty            (l2_tx_empty),
        .l2_tx_data             (l2_tx_data),

        .tx_lanes_stable        (tx_lanes_stable),
        .rx_block_lock          (rx_block_lock),
        .rx_am_lock             (rx_am_lock),
        .rx_pcs_ready           (rx_pcs_ready),
        .clk_rxmac              (clk_rxmac),

        .l2_txstatus_valid      (l2_txstatus_valid),    // 1  bits
        .l2_txstatus_data       (l2_txstatus_data),    // 40 bits
        .l2_txstatus_error      (l2_txstatus_error),    // 7  bits
        .l2_rxstatus_valid      (l2_rxstatus_valid),    // 1  bits
        .l2_rxstatus_data       (l2_rxstatus_data),    // 40 bits

        .l2_rx_error            (l2_rx_error),
        .l2_rx_valid            (l2_rx_valid),
        .l2_rx_startofpacket    (l2_rx_startofpacket),
        .l2_rx_endofpacket      (l2_rx_endofpacket),
        .l2_rx_data             (l2_rx_data),
        .l2_rx_empty            (l2_rx_empty),

        .reconfig_clk           (reconfig_clk),
        .reconfig_reset         (reconfig_reset),
        .reconfig_write         (reconfig_write),
        .reconfig_read          (reconfig_read),
        .reconfig_address       (reconfig_address),
        .reconfig_writedata     (reconfig_writedata),
        .reconfig_readdata      (reconfig_readdata),
        .reconfig_waitrequest   (reconfig_waitrequest)
    );

    defparam dut.alt_e40_0.SIM_SHORT_AM       = 1'b1;


    s10_atx_pll atx (
        .pll_refclk0    (clk_ref),
        .tx_serial_clk  (tx_serial_clk),
        .pll_locked     (tx_pll_locked),
        .pll_cal_busy   (/* Unused */)
    );

///// 2.0ms timeout    
initial begin
 #2ms
 $display ("Test did not finish. Timing out at 2ms"); 
 $finish();
end
endmodule

